"""
identical_meshes
================
Autogenerated DPF operator classes.
"""
from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification


class identical_meshes(Operator):
    """Takes two meshes and compares them. Note: When comparing mesh
    properties, the current behaviour is to verify that the properties
    in the first mesh (pin 0) are included in the second mesh (pin 1).

    Parameters
    ----------
    meshA : MeshedRegion
    meshB : MeshedRegion
    small_value : float
        Define what is a small value for numeric
        comparison.
    tolerance : float
        Define the relative tolerance ceil for
        numeric comparison.
    compare_auxiliary : bool
        Compare auxiliary data (i.e property fields,
        scopings...). default value is
        'false'.


    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.logic.identical_meshes()

    >>> # Make input connections
    >>> my_meshA = dpf.MeshedRegion()
    >>> op.inputs.meshA.connect(my_meshA)
    >>> my_meshB = dpf.MeshedRegion()
    >>> op.inputs.meshB.connect(my_meshB)
    >>> my_small_value = float()
    >>> op.inputs.small_value.connect(my_small_value)
    >>> my_tolerance = float()
    >>> op.inputs.tolerance.connect(my_tolerance)
    >>> my_compare_auxiliary = bool()
    >>> op.inputs.compare_auxiliary.connect(my_compare_auxiliary)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.logic.identical_meshes(
    ...     meshA=my_meshA,
    ...     meshB=my_meshB,
    ...     small_value=my_small_value,
    ...     tolerance=my_tolerance,
    ...     compare_auxiliary=my_compare_auxiliary,
    ... )

    >>> # Get output data
    >>> result_are_identical = op.outputs.are_identical()
    """

    def __init__(
        self,
        meshA=None,
        meshB=None,
        small_value=None,
        tolerance=None,
        compare_auxiliary=None,
        config=None,
        server=None,
    ):
        super().__init__(name="compare::mesh", config=config, server=server)
        self._inputs = InputsIdenticalMeshes(self)
        self._outputs = OutputsIdenticalMeshes(self)
        if meshA is not None:
            self.inputs.meshA.connect(meshA)
        if meshB is not None:
            self.inputs.meshB.connect(meshB)
        if small_value is not None:
            self.inputs.small_value.connect(small_value)
        if tolerance is not None:
            self.inputs.tolerance.connect(tolerance)
        if compare_auxiliary is not None:
            self.inputs.compare_auxiliary.connect(compare_auxiliary)

    @staticmethod
    def _spec():
        description = """Takes two meshes and compares them. Note: When comparing mesh
            properties, the current behaviour is to verify that the
            properties in the first mesh (pin 0) are included in the
            second mesh (pin 1)."""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                0: PinSpecification(
                    name="meshA",
                    type_names=["abstract_meshed_region"],
                    optional=False,
                    document="""""",
                ),
                1: PinSpecification(
                    name="meshB",
                    type_names=["abstract_meshed_region"],
                    optional=False,
                    document="""""",
                ),
                2: PinSpecification(
                    name="small_value",
                    type_names=["double"],
                    optional=False,
                    document="""Define what is a small value for numeric
        comparison.""",
                ),
                3: PinSpecification(
                    name="tolerance",
                    type_names=["double"],
                    optional=False,
                    document="""Define the relative tolerance ceil for
        numeric comparison.""",
                ),
                4: PinSpecification(
                    name="compare_auxiliary",
                    type_names=["bool"],
                    optional=False,
                    document="""Compare auxiliary data (i.e property fields,
        scopings...). default value is
        'false'.""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="are_identical",
                    type_names=["bool"],
                    optional=False,
                    document="""""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server=None):
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server : server.DPFServer, optional
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.
        """
        return Operator.default_config(name="compare::mesh", server=server)

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsIdenticalMeshes
        """
        return super().inputs

    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs : OutputsIdenticalMeshes
        """
        return super().outputs


class InputsIdenticalMeshes(_Inputs):
    """Intermediate class used to connect user inputs to
    identical_meshes operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.logic.identical_meshes()
    >>> my_meshA = dpf.MeshedRegion()
    >>> op.inputs.meshA.connect(my_meshA)
    >>> my_meshB = dpf.MeshedRegion()
    >>> op.inputs.meshB.connect(my_meshB)
    >>> my_small_value = float()
    >>> op.inputs.small_value.connect(my_small_value)
    >>> my_tolerance = float()
    >>> op.inputs.tolerance.connect(my_tolerance)
    >>> my_compare_auxiliary = bool()
    >>> op.inputs.compare_auxiliary.connect(my_compare_auxiliary)
    """

    def __init__(self, op: Operator):
        super().__init__(identical_meshes._spec().inputs, op)
        self._meshA = Input(identical_meshes._spec().input_pin(0), 0, op, -1)
        self._inputs.append(self._meshA)
        self._meshB = Input(identical_meshes._spec().input_pin(1), 1, op, -1)
        self._inputs.append(self._meshB)
        self._small_value = Input(identical_meshes._spec().input_pin(2), 2, op, -1)
        self._inputs.append(self._small_value)
        self._tolerance = Input(identical_meshes._spec().input_pin(3), 3, op, -1)
        self._inputs.append(self._tolerance)
        self._compare_auxiliary = Input(
            identical_meshes._spec().input_pin(4), 4, op, -1
        )
        self._inputs.append(self._compare_auxiliary)

    @property
    def meshA(self):
        """Allows to connect meshA input to the operator.

        Parameters
        ----------
        my_meshA : MeshedRegion

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.logic.identical_meshes()
        >>> op.inputs.meshA.connect(my_meshA)
        >>> # or
        >>> op.inputs.meshA(my_meshA)
        """
        return self._meshA

    @property
    def meshB(self):
        """Allows to connect meshB input to the operator.

        Parameters
        ----------
        my_meshB : MeshedRegion

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.logic.identical_meshes()
        >>> op.inputs.meshB.connect(my_meshB)
        >>> # or
        >>> op.inputs.meshB(my_meshB)
        """
        return self._meshB

    @property
    def small_value(self):
        """Allows to connect small_value input to the operator.

        Define what is a small value for numeric
        comparison.

        Parameters
        ----------
        my_small_value : float

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.logic.identical_meshes()
        >>> op.inputs.small_value.connect(my_small_value)
        >>> # or
        >>> op.inputs.small_value(my_small_value)
        """
        return self._small_value

    @property
    def tolerance(self):
        """Allows to connect tolerance input to the operator.

        Define the relative tolerance ceil for
        numeric comparison.

        Parameters
        ----------
        my_tolerance : float

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.logic.identical_meshes()
        >>> op.inputs.tolerance.connect(my_tolerance)
        >>> # or
        >>> op.inputs.tolerance(my_tolerance)
        """
        return self._tolerance

    @property
    def compare_auxiliary(self):
        """Allows to connect compare_auxiliary input to the operator.

        Compare auxiliary data (i.e property fields,
        scopings...). default value is
        'false'.

        Parameters
        ----------
        my_compare_auxiliary : bool

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.logic.identical_meshes()
        >>> op.inputs.compare_auxiliary.connect(my_compare_auxiliary)
        >>> # or
        >>> op.inputs.compare_auxiliary(my_compare_auxiliary)
        """
        return self._compare_auxiliary


class OutputsIdenticalMeshes(_Outputs):
    """Intermediate class used to get outputs from
    identical_meshes operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.logic.identical_meshes()
    >>> # Connect inputs : op.inputs. ...
    >>> result_are_identical = op.outputs.are_identical()
    """

    def __init__(self, op: Operator):
        super().__init__(identical_meshes._spec().outputs, op)
        self._are_identical = Output(identical_meshes._spec().output_pin(0), 0, op)
        self._outputs.append(self._are_identical)

    @property
    def are_identical(self):
        """Allows to get are_identical output of the operator

        Returns
        ----------
        my_are_identical : bool

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.logic.identical_meshes()
        >>> # Connect inputs : op.inputs. ...
        >>> result_are_identical = op.outputs.are_identical()
        """  # noqa: E501
        return self._are_identical



"""
elements_facets_surfaces_over_time
==================================
Autogenerated DPF operator classes.
"""
from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification


class elements_facets_surfaces_over_time(Operator):
    """Calculates for a mesh, the surface of each element's facet over time
    for each specified time step. The output is a new mesh made with
    only surface elements.

    Parameters
    ----------
    scoping : Scoping, optional
    displacement : FieldsContainer, optional
        Displacement field's container.
    mesh : MeshedRegion, optional
        Mesh must be defined if the displacement
        field's container does not contain
        it, or if there is no displacement.


    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.geo.elements_facets_surfaces_over_time()

    >>> # Make input connections
    >>> my_scoping = dpf.Scoping()
    >>> op.inputs.scoping.connect(my_scoping)
    >>> my_displacement = dpf.FieldsContainer()
    >>> op.inputs.displacement.connect(my_displacement)
    >>> my_mesh = dpf.MeshedRegion()
    >>> op.inputs.mesh.connect(my_mesh)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.geo.elements_facets_surfaces_over_time(
    ...     scoping=my_scoping,
    ...     displacement=my_displacement,
    ...     mesh=my_mesh,
    ... )

    >>> # Get output data
    >>> result_fields_container = op.outputs.fields_container()
    >>> result_mesh = op.outputs.mesh()
    """

    def __init__(
        self, scoping=None, displacement=None, mesh=None, config=None, server=None
    ):
        super().__init__(name="surfaces_provider", config=config, server=server)
        self._inputs = InputsElementsFacetsSurfacesOverTime(self)
        self._outputs = OutputsElementsFacetsSurfacesOverTime(self)
        if scoping is not None:
            self.inputs.scoping.connect(scoping)
        if displacement is not None:
            self.inputs.displacement.connect(displacement)
        if mesh is not None:
            self.inputs.mesh.connect(mesh)

    @staticmethod
    def _spec():
        description = """Calculates for a mesh, the surface of each element's facet over time
            for each specified time step. The output is a new mesh
            made with only surface elements."""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                1: PinSpecification(
                    name="scoping",
                    type_names=["scoping"],
                    optional=True,
                    document="""""",
                ),
                2: PinSpecification(
                    name="displacement",
                    type_names=["fields_container"],
                    optional=True,
                    document="""Displacement field's container.""",
                ),
                7: PinSpecification(
                    name="mesh",
                    type_names=["abstract_meshed_region"],
                    optional=True,
                    document="""Mesh must be defined if the displacement
        field's container does not contain
        it, or if there is no displacement.""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="fields_container",
                    type_names=["fields_container"],
                    optional=False,
                    document="""Surfaces field.""",
                ),
                1: PinSpecification(
                    name="mesh",
                    type_names=["abstract_meshed_region"],
                    optional=False,
                    document="""Mesh made of surface elements only.""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server=None):
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server : server.DPFServer, optional
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.
        """
        return Operator.default_config(name="surfaces_provider", server=server)

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsElementsFacetsSurfacesOverTime
        """
        return super().inputs

    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs : OutputsElementsFacetsSurfacesOverTime
        """
        return super().outputs


class InputsElementsFacetsSurfacesOverTime(_Inputs):
    """Intermediate class used to connect user inputs to
    elements_facets_surfaces_over_time operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.geo.elements_facets_surfaces_over_time()
    >>> my_scoping = dpf.Scoping()
    >>> op.inputs.scoping.connect(my_scoping)
    >>> my_displacement = dpf.FieldsContainer()
    >>> op.inputs.displacement.connect(my_displacement)
    >>> my_mesh = dpf.MeshedRegion()
    >>> op.inputs.mesh.connect(my_mesh)
    """

    def __init__(self, op: Operator):
        super().__init__(elements_facets_surfaces_over_time._spec().inputs, op)
        self._scoping = Input(
            elements_facets_surfaces_over_time._spec().input_pin(1), 1, op, -1
        )
        self._inputs.append(self._scoping)
        self._displacement = Input(
            elements_facets_surfaces_over_time._spec().input_pin(2), 2, op, -1
        )
        self._inputs.append(self._displacement)
        self._mesh = Input(
            elements_facets_surfaces_over_time._spec().input_pin(7), 7, op, -1
        )
        self._inputs.append(self._mesh)

    @property
    def scoping(self):
        """Allows to connect scoping input to the operator.

        Parameters
        ----------
        my_scoping : Scoping

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.geo.elements_facets_surfaces_over_time()
        >>> op.inputs.scoping.connect(my_scoping)
        >>> # or
        >>> op.inputs.scoping(my_scoping)
        """
        return self._scoping

    @property
    def displacement(self):
        """Allows to connect displacement input to the operator.

        Displacement field's container.

        Parameters
        ----------
        my_displacement : FieldsContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.geo.elements_facets_surfaces_over_time()
        >>> op.inputs.displacement.connect(my_displacement)
        >>> # or
        >>> op.inputs.displacement(my_displacement)
        """
        return self._displacement

    @property
    def mesh(self):
        """Allows to connect mesh input to the operator.

        Mesh must be defined if the displacement
        field's container does not contain
        it, or if there is no displacement.

        Parameters
        ----------
        my_mesh : MeshedRegion

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.geo.elements_facets_surfaces_over_time()
        >>> op.inputs.mesh.connect(my_mesh)
        >>> # or
        >>> op.inputs.mesh(my_mesh)
        """
        return self._mesh


class OutputsElementsFacetsSurfacesOverTime(_Outputs):
    """Intermediate class used to get outputs from
    elements_facets_surfaces_over_time operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.geo.elements_facets_surfaces_over_time()
    >>> # Connect inputs : op.inputs. ...
    >>> result_fields_container = op.outputs.fields_container()
    >>> result_mesh = op.outputs.mesh()
    """

    def __init__(self, op: Operator):
        super().__init__(elements_facets_surfaces_over_time._spec().outputs, op)
        self._fields_container = Output(
            elements_facets_surfaces_over_time._spec().output_pin(0), 0, op
        )
        self._outputs.append(self._fields_container)
        self._mesh = Output(
            elements_facets_surfaces_over_time._spec().output_pin(1), 1, op
        )
        self._outputs.append(self._mesh)

    @property
    def fields_container(self):
        """Allows to get fields_container output of the operator

        Returns
        ----------
        my_fields_container : FieldsContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.geo.elements_facets_surfaces_over_time()
        >>> # Connect inputs : op.inputs. ...
        >>> result_fields_container = op.outputs.fields_container()
        """  # noqa: E501
        return self._fields_container

    @property
    def mesh(self):
        """Allows to get mesh output of the operator

        Returns
        ----------
        my_mesh : MeshedRegion

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.geo.elements_facets_surfaces_over_time()
        >>> # Connect inputs : op.inputs. ...
        >>> result_mesh = op.outputs.mesh()
        """  # noqa: E501
        return self._mesh



"""
compute_total_strain_XY
=======================
Autogenerated DPF operator classes.
"""
from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification


class compute_total_strain_XY(Operator):
    """Computes the strain from a displacement field. Only SOLID185 (B-Bar,
    Simplified Enhanced Strain, Enhanced Strain formulations),
    SOLID186 (Full Integration) & SOLID187 elements are supported.
    Layered elements are not supported. Thermal strains are not
    supported. Only one value of material properties are allowed per
    element for isotropic and orthotropic elasticity. Material
    nonlinearity is not supported Only linear analysis are supported
    without On Demand Expansion. All coordinates are global
    coordinates. Euler Angles need to be included in the database. Get
    the XY shear component (01 component).

    Parameters
    ----------
    time_scoping : Scoping or int or float or Field, optional
        Time/freq values (use doubles or field),
        time/freq set ids (use ints or
        scoping) or time/freq step ids (use
        scoping with timefreq_steps location)
        required in output.to specify
        time/freq values at specific load
        steps, put a field (and not a list)
        in input with a scoping located on
        "timefreq_steps".linear time freq
        intrapolation is performed if the
        values are not in the result files
        and the data at the max time or freq
        is taken when time/freqs are higher
        than available time/freqs in result
        files.. will only be used if no
        displacement input is given (will be
        applied on displacement operator).
    scoping : Scoping, optional
        The element scoping on which the result is
        computed.
    streams_container : StreamsContainer, optional
        Optional if a mesh or a data_sources have
        been connected. required if no
        displacement input have been
        connected.
    data_sources : DataSources
        Optional if a mesh or a streams_container
        have been connected, or if the
        displacement's field has a mesh
        support. required if no displacement
        input have been connected.
    extrapolate : int, optional
        Whether to extrapolate the data from the
        integration points to the nodes.
    nonlinear : int, optional
        Whether to use nonlinear geometry or
        nonlinear material (1 = large strain,
        2 = hyperelasticity).
    abstract_meshed_region : MeshedRegion, optional
        The underlying mesh. optional if a
        data_sources or a streams_container
        have been connected, or if the
        displacement's field has a mesh
        support.
    requested_location : str, optional
        Average the elemental nodal result to the
        requested location.
    displacement : FieldsContainer or Field, optional
        Field/or fields container containing only the
        displacement field (nodal). if none
        specified, read displacements from
        result file using the data_sources.


    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.result.compute_total_strain_XY()

    >>> # Make input connections
    >>> my_time_scoping = dpf.Scoping()
    >>> op.inputs.time_scoping.connect(my_time_scoping)
    >>> my_scoping = dpf.Scoping()
    >>> op.inputs.scoping.connect(my_scoping)
    >>> my_streams_container = dpf.StreamsContainer()
    >>> op.inputs.streams_container.connect(my_streams_container)
    >>> my_data_sources = dpf.DataSources()
    >>> op.inputs.data_sources.connect(my_data_sources)
    >>> my_extrapolate = int()
    >>> op.inputs.extrapolate.connect(my_extrapolate)
    >>> my_nonlinear = int()
    >>> op.inputs.nonlinear.connect(my_nonlinear)
    >>> my_abstract_meshed_region = dpf.MeshedRegion()
    >>> op.inputs.abstract_meshed_region.connect(my_abstract_meshed_region)
    >>> my_requested_location = str()
    >>> op.inputs.requested_location.connect(my_requested_location)
    >>> my_displacement = dpf.FieldsContainer()
    >>> op.inputs.displacement.connect(my_displacement)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.result.compute_total_strain_XY(
    ...     time_scoping=my_time_scoping,
    ...     scoping=my_scoping,
    ...     streams_container=my_streams_container,
    ...     data_sources=my_data_sources,
    ...     extrapolate=my_extrapolate,
    ...     nonlinear=my_nonlinear,
    ...     abstract_meshed_region=my_abstract_meshed_region,
    ...     requested_location=my_requested_location,
    ...     displacement=my_displacement,
    ... )

    >>> # Get output data
    >>> result_fields_container = op.outputs.fields_container()
    """

    def __init__(
        self,
        time_scoping=None,
        scoping=None,
        streams_container=None,
        data_sources=None,
        extrapolate=None,
        nonlinear=None,
        abstract_meshed_region=None,
        requested_location=None,
        displacement=None,
        config=None,
        server=None,
    ):
        super().__init__(name="compute_total_strain_XY", config=config, server=server)
        self._inputs = InputsComputeTotalStrainXy(self)
        self._outputs = OutputsComputeTotalStrainXy(self)
        if time_scoping is not None:
            self.inputs.time_scoping.connect(time_scoping)
        if scoping is not None:
            self.inputs.scoping.connect(scoping)
        if streams_container is not None:
            self.inputs.streams_container.connect(streams_container)
        if data_sources is not None:
            self.inputs.data_sources.connect(data_sources)
        if extrapolate is not None:
            self.inputs.extrapolate.connect(extrapolate)
        if nonlinear is not None:
            self.inputs.nonlinear.connect(nonlinear)
        if abstract_meshed_region is not None:
            self.inputs.abstract_meshed_region.connect(abstract_meshed_region)
        if requested_location is not None:
            self.inputs.requested_location.connect(requested_location)
        if displacement is not None:
            self.inputs.displacement.connect(displacement)

    @staticmethod
    def _spec():
        description = """Computes the strain from a displacement field. Only SOLID185 (B-Bar,
            Simplified Enhanced Strain, Enhanced Strain formulations),
            SOLID186 (Full Integration) &amp; SOLID187 elements are
            supported. Layered elements are not supported. Thermal
            strains are not supported. Only one value of material
            properties are allowed per element for isotropic and
            orthotropic elasticity. Material nonlinearity is not
            supported Only linear analysis are supported without On
            Demand Expansion. All coordinates are global coordinates.
            Euler Angles need to be included in the database. Get the
            XY shear component (01 component)."""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                0: PinSpecification(
                    name="time_scoping",
                    type_names=[
                        "scoping",
                        "int32",
                        "vector<int32>",
                        "double",
                        "field",
                        "vector<double>",
                    ],
                    optional=True,
                    document="""Time/freq values (use doubles or field),
        time/freq set ids (use ints or
        scoping) or time/freq step ids (use
        scoping with timefreq_steps location)
        required in output.to specify
        time/freq values at specific load
        steps, put a field (and not a list)
        in input with a scoping located on
        "timefreq_steps".linear time freq
        intrapolation is performed if the
        values are not in the result files
        and the data at the max time or freq
        is taken when time/freqs are higher
        than available time/freqs in result
        files.. will only be used if no
        displacement input is given (will be
        applied on displacement operator).""",
                ),
                1: PinSpecification(
                    name="scoping",
                    type_names=["scoping"],
                    optional=True,
                    document="""The element scoping on which the result is
        computed.""",
                ),
                3: PinSpecification(
                    name="streams_container",
                    type_names=["streams_container"],
                    optional=True,
                    document="""Optional if a mesh or a data_sources have
        been connected. required if no
        displacement input have been
        connected.""",
                ),
                4: PinSpecification(
                    name="data_sources",
                    type_names=["data_sources"],
                    optional=False,
                    document="""Optional if a mesh or a streams_container
        have been connected, or if the
        displacement's field has a mesh
        support. required if no displacement
        input have been connected.""",
                ),
                5: PinSpecification(
                    name="extrapolate",
                    type_names=["int32"],
                    optional=True,
                    document="""Whether to extrapolate the data from the
        integration points to the nodes.""",
                ),
                6: PinSpecification(
                    name="nonlinear",
                    type_names=["int32"],
                    optional=True,
                    document="""Whether to use nonlinear geometry or
        nonlinear material (1 = large strain,
        2 = hyperelasticity).""",
                ),
                7: PinSpecification(
                    name="abstract_meshed_region",
                    type_names=["abstract_meshed_region"],
                    optional=True,
                    document="""The underlying mesh. optional if a
        data_sources or a streams_container
        have been connected, or if the
        displacement's field has a mesh
        support.""",
                ),
                9: PinSpecification(
                    name="requested_location",
                    type_names=["string"],
                    optional=True,
                    document="""Average the elemental nodal result to the
        requested location.""",
                ),
                10: PinSpecification(
                    name="displacement",
                    type_names=["fields_container", "field"],
                    optional=True,
                    document="""Field/or fields container containing only the
        displacement field (nodal). if none
        specified, read displacements from
        result file using the data_sources.""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="fields_container",
                    type_names=["fields_container"],
                    optional=False,
                    document="""The computed result fields container
        (elemental nodal).""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server=None):
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server : server.DPFServer, optional
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.
        """
        return Operator.default_config(name="compute_total_strain_XY", server=server)

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsComputeTotalStrainXy
        """
        return super().inputs

    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs : OutputsComputeTotalStrainXy
        """
        return super().outputs


class InputsComputeTotalStrainXy(_Inputs):
    """Intermediate class used to connect user inputs to
    compute_total_strain_XY operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.result.compute_total_strain_XY()
    >>> my_time_scoping = dpf.Scoping()
    >>> op.inputs.time_scoping.connect(my_time_scoping)
    >>> my_scoping = dpf.Scoping()
    >>> op.inputs.scoping.connect(my_scoping)
    >>> my_streams_container = dpf.StreamsContainer()
    >>> op.inputs.streams_container.connect(my_streams_container)
    >>> my_data_sources = dpf.DataSources()
    >>> op.inputs.data_sources.connect(my_data_sources)
    >>> my_extrapolate = int()
    >>> op.inputs.extrapolate.connect(my_extrapolate)
    >>> my_nonlinear = int()
    >>> op.inputs.nonlinear.connect(my_nonlinear)
    >>> my_abstract_meshed_region = dpf.MeshedRegion()
    >>> op.inputs.abstract_meshed_region.connect(my_abstract_meshed_region)
    >>> my_requested_location = str()
    >>> op.inputs.requested_location.connect(my_requested_location)
    >>> my_displacement = dpf.FieldsContainer()
    >>> op.inputs.displacement.connect(my_displacement)
    """

    def __init__(self, op: Operator):
        super().__init__(compute_total_strain_XY._spec().inputs, op)
        self._time_scoping = Input(
            compute_total_strain_XY._spec().input_pin(0), 0, op, -1
        )
        self._inputs.append(self._time_scoping)
        self._scoping = Input(compute_total_strain_XY._spec().input_pin(1), 1, op, -1)
        self._inputs.append(self._scoping)
        self._streams_container = Input(
            compute_total_strain_XY._spec().input_pin(3), 3, op, -1
        )
        self._inputs.append(self._streams_container)
        self._data_sources = Input(
            compute_total_strain_XY._spec().input_pin(4), 4, op, -1
        )
        self._inputs.append(self._data_sources)
        self._extrapolate = Input(
            compute_total_strain_XY._spec().input_pin(5), 5, op, -1
        )
        self._inputs.append(self._extrapolate)
        self._nonlinear = Input(compute_total_strain_XY._spec().input_pin(6), 6, op, -1)
        self._inputs.append(self._nonlinear)
        self._abstract_meshed_region = Input(
            compute_total_strain_XY._spec().input_pin(7), 7, op, -1
        )
        self._inputs.append(self._abstract_meshed_region)
        self._requested_location = Input(
            compute_total_strain_XY._spec().input_pin(9), 9, op, -1
        )
        self._inputs.append(self._requested_location)
        self._displacement = Input(
            compute_total_strain_XY._spec().input_pin(10), 10, op, -1
        )
        self._inputs.append(self._displacement)

    @property
    def time_scoping(self):
        """Allows to connect time_scoping input to the operator.

        Time/freq values (use doubles or field),
        time/freq set ids (use ints or
        scoping) or time/freq step ids (use
        scoping with timefreq_steps location)
        required in output.to specify
        time/freq values at specific load
        steps, put a field (and not a list)
        in input with a scoping located on
        "timefreq_steps".linear time freq
        intrapolation is performed if the
        values are not in the result files
        and the data at the max time or freq
        is taken when time/freqs are higher
        than available time/freqs in result
        files.. will only be used if no
        displacement input is given (will be
        applied on displacement operator).

        Parameters
        ----------
        my_time_scoping : Scoping or int or float or Field

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.compute_total_strain_XY()
        >>> op.inputs.time_scoping.connect(my_time_scoping)
        >>> # or
        >>> op.inputs.time_scoping(my_time_scoping)
        """
        return self._time_scoping

    @property
    def scoping(self):
        """Allows to connect scoping input to the operator.

        The element scoping on which the result is
        computed.

        Parameters
        ----------
        my_scoping : Scoping

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.compute_total_strain_XY()
        >>> op.inputs.scoping.connect(my_scoping)
        >>> # or
        >>> op.inputs.scoping(my_scoping)
        """
        return self._scoping

    @property
    def streams_container(self):
        """Allows to connect streams_container input to the operator.

        Optional if a mesh or a data_sources have
        been connected. required if no
        displacement input have been
        connected.

        Parameters
        ----------
        my_streams_container : StreamsContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.compute_total_strain_XY()
        >>> op.inputs.streams_container.connect(my_streams_container)
        >>> # or
        >>> op.inputs.streams_container(my_streams_container)
        """
        return self._streams_container

    @property
    def data_sources(self):
        """Allows to connect data_sources input to the operator.

        Optional if a mesh or a streams_container
        have been connected, or if the
        displacement's field has a mesh
        support. required if no displacement
        input have been connected.

        Parameters
        ----------
        my_data_sources : DataSources

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.compute_total_strain_XY()
        >>> op.inputs.data_sources.connect(my_data_sources)
        >>> # or
        >>> op.inputs.data_sources(my_data_sources)
        """
        return self._data_sources

    @property
    def extrapolate(self):
        """Allows to connect extrapolate input to the operator.

        Whether to extrapolate the data from the
        integration points to the nodes.

        Parameters
        ----------
        my_extrapolate : int

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.compute_total_strain_XY()
        >>> op.inputs.extrapolate.connect(my_extrapolate)
        >>> # or
        >>> op.inputs.extrapolate(my_extrapolate)
        """
        return self._extrapolate

    @property
    def nonlinear(self):
        """Allows to connect nonlinear input to the operator.

        Whether to use nonlinear geometry or
        nonlinear material (1 = large strain,
        2 = hyperelasticity).

        Parameters
        ----------
        my_nonlinear : int

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.compute_total_strain_XY()
        >>> op.inputs.nonlinear.connect(my_nonlinear)
        >>> # or
        >>> op.inputs.nonlinear(my_nonlinear)
        """
        return self._nonlinear

    @property
    def abstract_meshed_region(self):
        """Allows to connect abstract_meshed_region input to the operator.

        The underlying mesh. optional if a
        data_sources or a streams_container
        have been connected, or if the
        displacement's field has a mesh
        support.

        Parameters
        ----------
        my_abstract_meshed_region : MeshedRegion

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.compute_total_strain_XY()
        >>> op.inputs.abstract_meshed_region.connect(my_abstract_meshed_region)
        >>> # or
        >>> op.inputs.abstract_meshed_region(my_abstract_meshed_region)
        """
        return self._abstract_meshed_region

    @property
    def requested_location(self):
        """Allows to connect requested_location input to the operator.

        Average the elemental nodal result to the
        requested location.

        Parameters
        ----------
        my_requested_location : str

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.compute_total_strain_XY()
        >>> op.inputs.requested_location.connect(my_requested_location)
        >>> # or
        >>> op.inputs.requested_location(my_requested_location)
        """
        return self._requested_location

    @property
    def displacement(self):
        """Allows to connect displacement input to the operator.

        Field/or fields container containing only the
        displacement field (nodal). if none
        specified, read displacements from
        result file using the data_sources.

        Parameters
        ----------
        my_displacement : FieldsContainer or Field

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.compute_total_strain_XY()
        >>> op.inputs.displacement.connect(my_displacement)
        >>> # or
        >>> op.inputs.displacement(my_displacement)
        """
        return self._displacement


class OutputsComputeTotalStrainXy(_Outputs):
    """Intermediate class used to get outputs from
    compute_total_strain_XY operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.result.compute_total_strain_XY()
    >>> # Connect inputs : op.inputs. ...
    >>> result_fields_container = op.outputs.fields_container()
    """

    def __init__(self, op: Operator):
        super().__init__(compute_total_strain_XY._spec().outputs, op)
        self._fields_container = Output(
            compute_total_strain_XY._spec().output_pin(0), 0, op
        )
        self._outputs.append(self._fields_container)

    @property
    def fields_container(self):
        """Allows to get fields_container output of the operator

        Returns
        ----------
        my_fields_container : FieldsContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.compute_total_strain_XY()
        >>> # Connect inputs : op.inputs. ...
        >>> result_fields_container = op.outputs.fields_container()
        """  # noqa: E501
        return self._fields_container



"""
default_value
=============
Autogenerated DPF operator classes.
"""
from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification


class default_value(Operator):
    """Default return value from input pin 1 to output pin 0 if there is
    nothing on input pin 0.

    Parameters
    ----------
    forced_value : optional
    default_value :


    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.utility.default_value()

    >>> # Make input connections
    >>> my_forced_value = dpf.()
    >>> op.inputs.forced_value.connect(my_forced_value)
    >>> my_default_value = dpf.()
    >>> op.inputs.default_value.connect(my_default_value)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.utility.default_value(
    ...     forced_value=my_forced_value,
    ...     default_value=my_default_value,
    ... )

    >>> # Get output data
    >>> result_output = op.outputs.output()
    """

    def __init__(self, forced_value=None, default_value=None, config=None, server=None):
        super().__init__(name="default_value", config=config, server=server)
        self._inputs = InputsDefaultValue(self)
        self._outputs = OutputsDefaultValue(self)
        if forced_value is not None:
            self.inputs.forced_value.connect(forced_value)
        if default_value is not None:
            self.inputs.default_value.connect(default_value)

    @staticmethod
    def _spec():
        description = """Default return value from input pin 1 to output pin 0 if there is
            nothing on input pin 0."""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                0: PinSpecification(
                    name="forced_value",
                    type_names=["any"],
                    optional=True,
                    document="""""",
                ),
                1: PinSpecification(
                    name="default_value",
                    type_names=["any"],
                    optional=False,
                    document="""""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="output",
                    optional=False,
                    document="""""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server=None):
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server : server.DPFServer, optional
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.
        """
        return Operator.default_config(name="default_value", server=server)

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsDefaultValue
        """
        return super().inputs

    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs : OutputsDefaultValue
        """
        return super().outputs


class InputsDefaultValue(_Inputs):
    """Intermediate class used to connect user inputs to
    default_value operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.utility.default_value()
    >>> my_forced_value = dpf.()
    >>> op.inputs.forced_value.connect(my_forced_value)
    >>> my_default_value = dpf.()
    >>> op.inputs.default_value.connect(my_default_value)
    """

    def __init__(self, op: Operator):
        super().__init__(default_value._spec().inputs, op)
        self._forced_value = Input(default_value._spec().input_pin(0), 0, op, -1)
        self._inputs.append(self._forced_value)
        self._default_value = Input(default_value._spec().input_pin(1), 1, op, -1)
        self._inputs.append(self._default_value)

    @property
    def forced_value(self):
        """Allows to connect forced_value input to the operator.

        Parameters
        ----------
        my_forced_value :

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.default_value()
        >>> op.inputs.forced_value.connect(my_forced_value)
        >>> # or
        >>> op.inputs.forced_value(my_forced_value)
        """
        return self._forced_value

    @property
    def default_value(self):
        """Allows to connect default_value input to the operator.

        Parameters
        ----------
        my_default_value :

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.default_value()
        >>> op.inputs.default_value.connect(my_default_value)
        >>> # or
        >>> op.inputs.default_value(my_default_value)
        """
        return self._default_value


class OutputsDefaultValue(_Outputs):
    """Intermediate class used to get outputs from
    default_value operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.utility.default_value()
    >>> # Connect inputs : op.inputs. ...
    >>> result_output = op.outputs.output()
    """

    def __init__(self, op: Operator):
        super().__init__(default_value._spec().outputs, op)
        self._output = Output(default_value._spec().output_pin(0), 0, op)
        self._outputs.append(self._output)

    @property
    def output(self):
        """Allows to get output output of the operator

        Returns
        ----------
        my_output :

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.default_value()
        >>> # Connect inputs : op.inputs. ...
        >>> result_output = op.outputs.output()
        """  # noqa: E501
        return self._output



"""
Collection
===========
Contains classes associated with the DPF collection.

"""
import abc
import warnings
import traceback

import numpy as np

from ansys.dpf.core.server_types import BaseServer
from ansys.dpf.core.scoping import Scoping
from ansys.dpf.core.label_space import LabelSpace
from ansys.dpf.core import server as server_module
from ansys.dpf.gate import (
    collection_capi,
    collection_grpcapi,
    data_processing_capi,
    data_processing_grpcapi,
    dpf_vector,
    dpf_array,
)


class Collection:
    """Represents a collection of entries ordered by labels and IDs.

    Parameters
    ----------
    dpf_type :

    collection : ansys.grpc.dpf.collection_pb2.Collection, optional
        Collection to create from the collection message. The default is ``None``.
    server : server.DPFServer, optional
        Server with the channel connected to the remote or local instance. The
        default is ``None``, in which case an attempt is made to use the global
        server.

    """

    def __init__(self, collection=None, server: BaseServer = None):
        # step 1: get server
        self._server = server_module.get_or_create_server(server)

        # step2: if object exists, take the instance, else create it
        self._internal_obj = None
        if collection is not None:
            if isinstance(collection, Collection):
                self._server = collection._server
                core_api = self._server.get_api_for_type(
                    capi=data_processing_capi.DataProcessingCAPI,
                    grpcapi=data_processing_grpcapi.DataProcessingGRPCAPI,
                )
                core_api.init_data_processing_environment(self)
                self._internal_obj = core_api.data_processing_duplicate_object_reference(collection)
            else:
                self._internal_obj = collection
        self.owned = False

    @property
    def _server(self):
        return self._server_instance

    @_server.setter
    def _server(self, value):
        self._server_instance = value
        # step 2: get api
        self._api = self._server.get_api_for_type(
            capi=collection_capi.CollectionCAPI,
            grpcapi=collection_grpcapi.CollectionGRPCAPI,
        )
        # step3: init environment
        self._api.init_collection_environment(self)  # creates stub when gRPC

    @abc.abstractmethod
    def create_subtype(self, obj_by_copy):
        pass

    @staticmethod
    def integral_collection(inpt, server: BaseServer = None):
        """Creates a collection of integral type with a list.

        The collection of integral is the equivalent of an array of
        data sent server side. It can be used to efficiently stream
        large data to the server.

        Parameters
        ----------
        inpt : list[float], list[int], numpy.array
            list to transfer server side

        Returns
        -------
        IntegralCollection

        Notes
        -----
        Used by default by the ``'Operator'`` and the``'Workflow'`` when a
        list is connected or returned.

        """
        if isinstance(inpt, np.ndarray):
            inpt = inpt.flatten()
        if all(isinstance(x, (int, np.int32)) for x in inpt):
            return IntCollection(inpt, server=server)
        if all(isinstance(x, (float, np.float64)) for x in inpt):
            return FloatCollection(inpt, server=server)
        else:
            raise NotImplementedError(
                f"{IntegralCollection.__name__} is only "
                "implemented for int and float values "
                f"and not {type(inpt[0]).__name__}"
            )

    def set_labels(self, labels):
        """Set labels for scoping the collection.

        Parameters
        ----------
        labels : list[str], optional
            Labels to scope entries to. For example, ``["time", "complex"]``.

        """
        current_labels = self.labels
        if len(current_labels) != 0:
            print(
                "The collection already has labels :",
                current_labels,
                "deleting existing labels is not implemented yet.",
            )
            return
        for label in labels:
            self.add_label(label)

    def add_label(self, label, default_value=None):
        """Add the requested label to scope the collection.

        Parameters
        ----------
        label : str
            Labels to scope the entries to. For example, ``"time"``.

        default_value : int, optional
            Default value for existing fields in the collection. The default
            is ``None``.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> coll = dpf.FieldsContainer()
        >>> coll.add_label('time')

        """
        if default_value is not None:
            self._api.collection_add_label_with_default_value(self, label, default_value)
        else:
            self._api.collection_add_label(self, label)

    def _get_labels(self):
        """Retrieve labels scoping the collection.

        Returns
        -------
        labels: list[str]
            List of labels that entries are scoped to. For example, ``["time", "complex"]``.
        """
        num = self._api.collection_get_num_labels(self)
        out = []
        for i in range(0, num):
            out.append(self._api.collection_get_label(self, i))
        return out

    labels = property(_get_labels, set_labels, "labels")

    def has_label(self, label):
        """Check if a collection has a specified label.

        Parameters
        ----------
        label: str
            Label to search for. For example, ``"time"``.

        Returns
        -------
        bool
           ``True`` when successful, ``False`` when failed.

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> coll = dpf.FieldsContainer()
        >>> coll.add_label('time')
        >>> coll.has_label('time')
        True

        >>> coll.has_label('complex')
        False

        """
        return label in self.labels

    def _get_entries(self, label_space_or_index):
        """Retrieve the entries at a requested label space or index.

        Parameters
        ----------
        label_space_or_index : dict[str,int]
            Label space or index. For example,
            ``{"time": 1, "complex": 0}`` or the index of the field.

        Returns
        -------
        entries : list[Scoping], list[Field], list[MeshedRegion]
            Entries corresponding to the request.
        """
        if isinstance(label_space_or_index, dict):
            client_label_space = LabelSpace(
                label_space=label_space_or_index, obj=self, server=self._server
            )
            num = self._api.collection_get_num_obj_for_label_space(self, client_label_space)
            out = []
            for i in range(0, num):
                out.append(
                    self.create_subtype(
                        self._api.collection_get_obj_by_index_for_label_space(
                            self, client_label_space, i
                        )
                    )
                )
            return out
        else:
            return self.create_subtype(
                self._api.collection_get_obj_by_index(self, label_space_or_index)
            )

    def _get_entry(self, label_space_or_index):
        """Retrieve the entry at a requested label space or index.

        Parameters
        ----------
        label_space_or_index : dict[str,int]
            Label space or index of the requested entry. For example,
            ``{"time": 1, "complex": 0}`` or the index of the field.

        Returns
        -------
        entry : Scoping, Field, MeshedRegion
            Entry at the requested label space or index.
        """
        entries = self._get_entries(label_space_or_index)
        if isinstance(entries, list):
            if len(entries) == 1:
                return entries[0]
            elif len(entries) == 0:
                return None
            else:
                raise KeyError(f"{label_space_or_index} has {len(entries)} entries")
        else:
            return entries

    def get_label_space(self, index):
        """Retrieve the label space of an entry at a requested index.

        Parameters
        ----------
        index: int
            Index of the entry.

        Returns
        -------
        label_space : dict(str:int)
            Scoping of the requested entry. For example,
            ``{"time": 1, "complex": 0}``.
        """
        return LabelSpace(
            label_space=self._api.collection_get_obj_label_space_by_index(self, index),
            server=self._server,
        ).__dict__()

    def get_available_ids_for_label(self, label="time"):
        """Retrieve the IDs assigned to an input label.

        Parameters
        ----------
        label : str
            Name of the input label. The default is ``"time"``.

        Returns
        -------
        ids : list[int]
            List of IDs assigned to the input label.
        """
        return self.get_label_scoping(label)._get_ids(False)

    def get_label_scoping(self, label="time"):
        """Retrieve the scoping for an input label.

        This method allows you to retrieve a list of IDs for a given input label in the
        collection. For example, if the label ``el_type`` exists in the collection, you
        can use the `get_lable_scoping` method to retrieve a list of IDS with this label.
        You can then use these IDs to request a given entity inside the collection.

        Parameters
        ----------
        label: str
            Name of the input label.

        Returns
        -------
        scoping: Scoping
            IDs scoped to the input label.
        """
        scoping = Scoping(self._api.collection_get_label_scoping(self, label), server=self._server)
        return scoping

    def __getitem__(self, index):
        """Retrieves the entry at a requested index value.

        Parameters
        ----------
        index : int
            Index value.

        Returns
        -------
        entry : Field , Scoping
            Entry at the index value.
        """
        self_len = len(self)
        if index < 0:
            # convert to a positive index
            index = self_len + index

        if not self_len:
            raise IndexError("This collection contains no items.")
        if index >= self_len:
            word = "entry" if self_len == 1 else "entries"
            raise IndexError(f"This collection contains only {self_len} {word}.")

        return self._get_entries(index)

    @property
    def _data_processing_core_api(self):
        core_api = self._server.get_api_for_type(
            capi=data_processing_capi.DataProcessingCAPI,
            grpcapi=data_processing_grpcapi.DataProcessingGRPCAPI,
        )
        core_api.init_data_processing_environment(self)
        return core_api

    def _add_entry(self, label_space, entry):
        """Update or add an entry at a requested label space.

        parameters
        ----------
        label_space : list[str,int]
            Label space of the requested fields. For example, ``{"time":1, "complex":0}``.
        entry : Field or Scoping
            DPF entry to add.
        """
        client_label_space = LabelSpace(label_space=label_space, obj=self, server=self._server)
        self._api.collection_add_entry(self, client_label_space, entry)

    def _get_time_freq_support(self):
        """Retrieve time frequency support.

        Returns
        -------
        time_freq_support : TimeFreqSupport
        """
        from ansys.dpf.core.time_freq_support import TimeFreqSupport
        from ansys.dpf.gate import (
            support_capi,
            support_grpcapi,
            object_handler,
            data_processing_capi,
            data_processing_grpcapi,
        )

        data_api = self._server.get_api_for_type(
            capi=data_processing_capi.DataProcessingCAPI,
            grpcapi=data_processing_grpcapi.DataProcessingGRPCAPI,
        )
        internal_obj = None
        # LegacyGrpcServer throws UNAVAILABLE when retrieving nullptr
        # This is all to obtain a None instead of throwing
        import ansys.dpf.gate.errors as err

        try:
            internal_obj = self._api.collection_get_support(self, "time")
        except err.DPFServerException as e:
            str_to_ignore = "The collection does not have a support."
            if str_to_ignore not in str(e):
                raise e
        if not internal_obj:
            return None

        support = object_handler.ObjHandler(
            data_processing_api=data_api,
            internal_obj=internal_obj,
            server=self._server,
        )
        support_api = self._server.get_api_for_type(
            capi=support_capi.SupportCAPI, grpcapi=support_grpcapi.SupportGRPCAPI
        )

        time_freq = support_api.support_get_as_time_freq_support(support)
        res = TimeFreqSupport(time_freq_support=time_freq, server=self._server)
        return res

    def _set_time_freq_support(self, time_freq_support):
        """Set the time frequency support of the collection."""
        self._api.collection_set_support(self, "time", time_freq_support)

    def __str__(self):
        """Describe the entity.

        Returns
        -------
        description : str
            Description of the entity.
        """
        from ansys.dpf.core.core import _description

        return _description(self._internal_obj, self._server)

    def __len__(self):
        """Retrieve the number of entries."""
        return self._api.collection_get_size(self)

    def __del__(self):
        """Delete the entry."""
        try:
            # delete
            if not self.owned:
                obj = self._deleter_func[1](self)
                if obj is not None:
                    self._deleter_func[0](obj)
        except:
            warnings.warn(traceback.format_exc())

    def _get_ownership(self):
        self.owned = True
        return self._internal_obj

    def __iter__(self):
        for i in range(len(self)):
            yield self[i]


class IntegralCollection(Collection):
    """Creates a collection of integral type with a list.

    The collection of integral is the equivalent of an array of
    data sent server side. It can be used to efficiently stream
    large data to the server.

    Parameters
    ----------
    list : list[float], list[int], numpy.array
        list to transfer server side

    Notes
    -----
    Used by default by the ``'Operator'`` and the``'Workflow'`` when a
    list is connected or returned.
    """

    def __init__(self, server=None, collection=None):
        super().__init__(server=server, collection=collection)

    @abc.abstractmethod
    def create_subtype(self, obj_by_copy):
        pass

    @abc.abstractmethod
    def _set_integral_entries(self, input):
        pass

    def get_integral_entries(self):
        pass


class IntCollection(Collection):
    """Creates a collection of integers with a list.

    The collection of integral is the equivalent of an array of
    data sent server side. It can be used to efficiently stream
    large data to the server.

    Parameters
    ----------
    list : list[int], numpy.array
        list to transfer server side

    Notes
    -----
    Used by default by the ``'Operator'`` and the``'Workflow'`` when a
    list is connected or returned.
    """

    def __init__(self, list=None, server=None, collection=None):
        super().__init__(server=server, collection=collection)
        if self._internal_obj is None:
            if self._server.has_client():
                self._internal_obj = self._api.collection_of_int_new_on_client(self._server.client)
            else:
                self._internal_obj = self._api.collection_of_int_new()
        if list is not None:
            self._set_integral_entries(list)

    def create_subtype(self, obj_by_copy):
        return int(obj_by_copy)

    def _set_integral_entries(self, input):
        dtype = np.int32
        if isinstance(input, range):
            input = np.array(list(input), dtype=dtype)
        elif not isinstance(input, (np.ndarray, np.generic)):
            input = np.array(input, dtype=dtype)
        else:
            input = np.array(list(input), dtype=dtype)

        self._api.collection_set_data_as_int(self, input, input.size)

    def get_integral_entries(self):
        try:
            vec = dpf_vector.DPFVectorInt(client=self._server.client)
            self._api.collection_get_data_as_int_for_dpf_vector(
                self, vec, vec.internal_data, vec.internal_size
            )
            return dpf_array.DPFArray(vec)
        except NotImplementedError:
            return self._api.collection_get_data_as_int(self, 0)


class FloatCollection(Collection):
    """Creates a collection of floats (double64) with a list.

    The collection of integral is the equivalent of an array of
    data sent server side. It can be used to efficiently stream
    large data to the server.

    Parameters
    ----------
    list : list[float], numpy.array
        list to transfer server side

    Notes
    -----
    Used by default by the ``'Operator'`` and the``'Workflow'`` when a
    list is connected or returned.
    """

    def __init__(self, list=None, server=None, collection=None):
        super().__init__(server=server, collection=collection)
        self._sub_type = float
        if self._internal_obj is None:
            if self._server.has_client():
                self._internal_obj = self._api.collection_of_double_new_on_client(
                    self._server.client
                )
            else:
                self._internal_obj = self._api.collection_of_double_new()
        if list is not None:
            self._set_integral_entries(list)

    def create_subtype(self, obj_by_copy):
        return float(obj_by_copy)

    def _set_integral_entries(self, input):
        dtype = np.float64
        if isinstance(input, range):
            input = np.array(list(input), dtype=dtype)
        elif not isinstance(input, (np.ndarray, np.generic)):
            input = np.array(input, dtype=dtype)
        else:
            input = np.array(list(input), dtype=dtype)

        self._api.collection_set_data_as_double(self, input, input.size)

    def get_integral_entries(self):
        try:
            vec = dpf_vector.DPFVectorDouble(client=self._server.client)
            self._api.collection_get_data_as_double_for_dpf_vector(
                self, vec, vec.internal_data, vec.internal_size
            )
            return dpf_array.DPFArray(vec)
        except NotImplementedError:
            return self._api.collection_get_data_as_double(self, 0)


class StringCollection(Collection):
    """Creates a collection of strings with a list.

    The collection of integral is the equivalent of an array of
    data sent server side. It can be used to efficiently stream
    large data to the server.

    Parameters
    ----------
    list : list[float], numpy.array
        list to transfer server side

    Notes
    -----
    Used by default by the ``'Operator'`` and the``'Workflow'`` when a
    list is connected or returned.
    """

    def __init__(self, list=None, server=None, collection=None, local: bool = False):
        super().__init__(server=server, collection=collection)
        self._sub_type = str
        if self._internal_obj is None:
            if self._server.has_client():
                if local:
                    self._internal_obj = self._api.collection_of_string_new_local(
                        self._server.client
                    )
                else:
                    self._internal_obj = self._api.collection_of_string_new_on_client(
                        self._server.client
                    )
            else:
                self._internal_obj = self._api.collection_of_string_new()
        if list is not None:
            self._set_integral_entries(list)

    def create_subtype(self, obj_by_copy):
        return str(obj_by_copy)

    def _set_integral_entries(self, input):
        for s in input:
            self._api.collection_add_string_entry(self, s)

    def get_integral_entries(self):
        num = self._api.collection_get_size(self)
        out = []
        for i in range(num):
            out.append(self._api.collection_get_string_entry(self, i))
        return out



"""
hdf5dpf_custom_read
===================
Autogenerated DPF operator classes.
"""
from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.outputs import _modify_output_spec_with_one_type
from ansys.dpf.core.operators.specification import PinSpecification, Specification


class hdf5dpf_custom_read(Operator):
    """Extract a custom result from an hdf5dpf file.

    Parameters
    ----------
    time_scoping : Scoping, optional
    mesh_scoping : Scoping, optional
    streams : StreamsContainer, optional
        Hdf5df file stream.
    data_sources : DataSources, optional
        Hdf5df file data source.
    meta_data : DataTree, optional
        Meta_data that may be used to evaluate
        results or extract workflows.
    result_name :
        Name of the result that must be extracted
        from the hdf5dpf file


    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.serialization.hdf5dpf_custom_read()

    >>> # Make input connections
    >>> my_time_scoping = dpf.Scoping()
    >>> op.inputs.time_scoping.connect(my_time_scoping)
    >>> my_mesh_scoping = dpf.Scoping()
    >>> op.inputs.mesh_scoping.connect(my_mesh_scoping)
    >>> my_streams = dpf.StreamsContainer()
    >>> op.inputs.streams.connect(my_streams)
    >>> my_data_sources = dpf.DataSources()
    >>> op.inputs.data_sources.connect(my_data_sources)
    >>> my_meta_data = dpf.DataTree()
    >>> op.inputs.meta_data.connect(my_meta_data)
    >>> my_result_name = dpf.()
    >>> op.inputs.result_name.connect(my_result_name)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.serialization.hdf5dpf_custom_read(
    ...     time_scoping=my_time_scoping,
    ...     mesh_scoping=my_mesh_scoping,
    ...     streams=my_streams,
    ...     data_sources=my_data_sources,
    ...     meta_data=my_meta_data,
    ...     result_name=my_result_name,
    ... )

    >>> # Get output data
    >>> result_field_or_fields_container = op.outputs.field_or_fields_container()
    """

    def __init__(
        self,
        time_scoping=None,
        mesh_scoping=None,
        streams=None,
        data_sources=None,
        meta_data=None,
        result_name=None,
        config=None,
        server=None,
    ):
        super().__init__(name="hdf5::h5dpf::custom", config=config, server=server)
        self._inputs = InputsHdf5DpfCustomRead(self)
        self._outputs = OutputsHdf5DpfCustomRead(self)
        if time_scoping is not None:
            self.inputs.time_scoping.connect(time_scoping)
        if mesh_scoping is not None:
            self.inputs.mesh_scoping.connect(mesh_scoping)
        if streams is not None:
            self.inputs.streams.connect(streams)
        if data_sources is not None:
            self.inputs.data_sources.connect(data_sources)
        if meta_data is not None:
            self.inputs.meta_data.connect(meta_data)
        if result_name is not None:
            self.inputs.result_name.connect(result_name)

    @staticmethod
    def _spec():
        description = """Extract a custom result from an hdf5dpf file."""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                0: PinSpecification(
                    name="time_scoping",
                    type_names=["scoping"],
                    optional=True,
                    document="""""",
                ),
                1: PinSpecification(
                    name="mesh_scoping",
                    type_names=["scoping"],
                    optional=True,
                    document="""""",
                ),
                3: PinSpecification(
                    name="streams",
                    type_names=["streams_container"],
                    optional=True,
                    document="""Hdf5df file stream.""",
                ),
                4: PinSpecification(
                    name="data_sources",
                    type_names=["data_sources"],
                    optional=True,
                    document="""Hdf5df file data source.""",
                ),
                24: PinSpecification(
                    name="meta_data",
                    type_names=["abstract_data_tree"],
                    optional=True,
                    document="""Meta_data that may be used to evaluate
        results or extract workflows.""",
                ),
                60: PinSpecification(
                    name="result_name",
                    type_names=["any"],
                    optional=False,
                    document="""Name of the result that must be extracted
        from the hdf5dpf file""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="field_or_fields_container",
                    type_names=["fields_container", "field"],
                    optional=False,
                    document="""""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server=None):
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server : server.DPFServer, optional
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.
        """
        return Operator.default_config(name="hdf5::h5dpf::custom", server=server)

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsHdf5DpfCustomRead
        """
        return super().inputs

    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs : OutputsHdf5DpfCustomRead
        """
        return super().outputs


class InputsHdf5DpfCustomRead(_Inputs):
    """Intermediate class used to connect user inputs to
    hdf5dpf_custom_read operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.serialization.hdf5dpf_custom_read()
    >>> my_time_scoping = dpf.Scoping()
    >>> op.inputs.time_scoping.connect(my_time_scoping)
    >>> my_mesh_scoping = dpf.Scoping()
    >>> op.inputs.mesh_scoping.connect(my_mesh_scoping)
    >>> my_streams = dpf.StreamsContainer()
    >>> op.inputs.streams.connect(my_streams)
    >>> my_data_sources = dpf.DataSources()
    >>> op.inputs.data_sources.connect(my_data_sources)
    >>> my_meta_data = dpf.DataTree()
    >>> op.inputs.meta_data.connect(my_meta_data)
    >>> my_result_name = dpf.()
    >>> op.inputs.result_name.connect(my_result_name)
    """

    def __init__(self, op: Operator):
        super().__init__(hdf5dpf_custom_read._spec().inputs, op)
        self._time_scoping = Input(hdf5dpf_custom_read._spec().input_pin(0), 0, op, -1)
        self._inputs.append(self._time_scoping)
        self._mesh_scoping = Input(hdf5dpf_custom_read._spec().input_pin(1), 1, op, -1)
        self._inputs.append(self._mesh_scoping)
        self._streams = Input(hdf5dpf_custom_read._spec().input_pin(3), 3, op, -1)
        self._inputs.append(self._streams)
        self._data_sources = Input(hdf5dpf_custom_read._spec().input_pin(4), 4, op, -1)
        self._inputs.append(self._data_sources)
        self._meta_data = Input(hdf5dpf_custom_read._spec().input_pin(24), 24, op, -1)
        self._inputs.append(self._meta_data)
        self._result_name = Input(hdf5dpf_custom_read._spec().input_pin(60), 60, op, -1)
        self._inputs.append(self._result_name)

    @property
    def time_scoping(self):
        """Allows to connect time_scoping input to the operator.

        Parameters
        ----------
        my_time_scoping : Scoping

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.serialization.hdf5dpf_custom_read()
        >>> op.inputs.time_scoping.connect(my_time_scoping)
        >>> # or
        >>> op.inputs.time_scoping(my_time_scoping)
        """
        return self._time_scoping

    @property
    def mesh_scoping(self):
        """Allows to connect mesh_scoping input to the operator.

        Parameters
        ----------
        my_mesh_scoping : Scoping

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.serialization.hdf5dpf_custom_read()
        >>> op.inputs.mesh_scoping.connect(my_mesh_scoping)
        >>> # or
        >>> op.inputs.mesh_scoping(my_mesh_scoping)
        """
        return self._mesh_scoping

    @property
    def streams(self):
        """Allows to connect streams input to the operator.

        Hdf5df file stream.

        Parameters
        ----------
        my_streams : StreamsContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.serialization.hdf5dpf_custom_read()
        >>> op.inputs.streams.connect(my_streams)
        >>> # or
        >>> op.inputs.streams(my_streams)
        """
        return self._streams

    @property
    def data_sources(self):
        """Allows to connect data_sources input to the operator.

        Hdf5df file data source.

        Parameters
        ----------
        my_data_sources : DataSources

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.serialization.hdf5dpf_custom_read()
        >>> op.inputs.data_sources.connect(my_data_sources)
        >>> # or
        >>> op.inputs.data_sources(my_data_sources)
        """
        return self._data_sources

    @property
    def meta_data(self):
        """Allows to connect meta_data input to the operator.

        Meta_data that may be used to evaluate
        results or extract workflows.

        Parameters
        ----------
        my_meta_data : DataTree

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.serialization.hdf5dpf_custom_read()
        >>> op.inputs.meta_data.connect(my_meta_data)
        >>> # or
        >>> op.inputs.meta_data(my_meta_data)
        """
        return self._meta_data

    @property
    def result_name(self):
        """Allows to connect result_name input to the operator.

        Name of the result that must be extracted
        from the hdf5dpf file

        Parameters
        ----------
        my_result_name :

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.serialization.hdf5dpf_custom_read()
        >>> op.inputs.result_name.connect(my_result_name)
        >>> # or
        >>> op.inputs.result_name(my_result_name)
        """
        return self._result_name


class OutputsHdf5DpfCustomRead(_Outputs):
    """Intermediate class used to get outputs from
    hdf5dpf_custom_read operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.serialization.hdf5dpf_custom_read()
    >>> # Connect inputs : op.inputs. ...
    >>> result_field_or_fields_container = op.outputs.field_or_fields_container()
    """

    def __init__(self, op: Operator):
        super().__init__(hdf5dpf_custom_read._spec().outputs, op)
        self.field_or_fields_container_as_fields_container = Output(
            _modify_output_spec_with_one_type(
                hdf5dpf_custom_read._spec().output_pin(0), "fields_container"
            ),
            0,
            op,
        )
        self._outputs.append(self.field_or_fields_container_as_fields_container)
        self.field_or_fields_container_as_field = Output(
            _modify_output_spec_with_one_type(
                hdf5dpf_custom_read._spec().output_pin(0), "field"
            ),
            0,
            op,
        )
        self._outputs.append(self.field_or_fields_container_as_field)



"""
serialize_to_hdf5
=================
Autogenerated DPF operator classes.
"""
from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification


class serialize_to_hdf5(Operator):
    """This operator is deprecated: use 'hdf5::h5dpf::make_result_file'
    instead. Serialize the inputs in an hdf5 format.

    Parameters
    ----------
    file_path : str
        Output file path with .h5 extension
    export_floats : bool, optional
        Converts double to float to reduce file size
        (default is true)
    export_flat_vectors : bool, optional
        If true, vectors and matrices data are
        exported flat (x1,y1,z1,x2,y2,z2..)
        (default is false)
    data1 :
        Only the data set explicitly to export is
        exported
    data2 :
        Only the data set explicitly to export is
        exported


    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.serialization.serialize_to_hdf5()

    >>> # Make input connections
    >>> my_file_path = str()
    >>> op.inputs.file_path.connect(my_file_path)
    >>> my_export_floats = bool()
    >>> op.inputs.export_floats.connect(my_export_floats)
    >>> my_export_flat_vectors = bool()
    >>> op.inputs.export_flat_vectors.connect(my_export_flat_vectors)
    >>> my_data1 = dpf.()
    >>> op.inputs.data1.connect(my_data1)
    >>> my_data2 = dpf.()
    >>> op.inputs.data2.connect(my_data2)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.serialization.serialize_to_hdf5(
    ...     file_path=my_file_path,
    ...     export_floats=my_export_floats,
    ...     export_flat_vectors=my_export_flat_vectors,
    ...     data1=my_data1,
    ...     data2=my_data2,
    ... )

    """

    def __init__(
        self,
        file_path=None,
        export_floats=None,
        export_flat_vectors=None,
        data1=None,
        data2=None,
        config=None,
        server=None,
    ):
        super().__init__(name="serialize_to_hdf5", config=config, server=server)
        self._inputs = InputsSerializeToHdf5(self)
        self._outputs = OutputsSerializeToHdf5(self)
        if file_path is not None:
            self.inputs.file_path.connect(file_path)
        if export_floats is not None:
            self.inputs.export_floats.connect(export_floats)
        if export_flat_vectors is not None:
            self.inputs.export_flat_vectors.connect(export_flat_vectors)
        if data1 is not None:
            self.inputs.data1.connect(data1)
        if data2 is not None:
            self.inputs.data2.connect(data2)

    @staticmethod
    def _spec():
        description = """This operator is deprecated: use 'hdf5::h5dpf::make_result_file'
            instead. Serialize the inputs in an hdf5 format."""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                0: PinSpecification(
                    name="file_path",
                    type_names=["string"],
                    optional=False,
                    document="""Output file path with .h5 extension""",
                ),
                1: PinSpecification(
                    name="export_floats",
                    type_names=["bool"],
                    optional=True,
                    document="""Converts double to float to reduce file size
        (default is true)""",
                ),
                2: PinSpecification(
                    name="export_flat_vectors",
                    type_names=["bool"],
                    optional=True,
                    document="""If true, vectors and matrices data are
        exported flat (x1,y1,z1,x2,y2,z2..)
        (default is false)""",
                ),
                3: PinSpecification(
                    name="data",
                    type_names=["any"],
                    optional=False,
                    document="""Only the data set explicitly to export is
        exported""",
                ),
                4: PinSpecification(
                    name="data",
                    type_names=["any"],
                    optional=False,
                    document="""Only the data set explicitly to export is
        exported""",
                ),
            },
            map_output_pin_spec={},
        )
        return spec

    @staticmethod
    def default_config(server=None):
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server : server.DPFServer, optional
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.
        """
        return Operator.default_config(name="serialize_to_hdf5", server=server)

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsSerializeToHdf5
        """
        return super().inputs

    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs : OutputsSerializeToHdf5
        """
        return super().outputs


class InputsSerializeToHdf5(_Inputs):
    """Intermediate class used to connect user inputs to
    serialize_to_hdf5 operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.serialization.serialize_to_hdf5()
    >>> my_file_path = str()
    >>> op.inputs.file_path.connect(my_file_path)
    >>> my_export_floats = bool()
    >>> op.inputs.export_floats.connect(my_export_floats)
    >>> my_export_flat_vectors = bool()
    >>> op.inputs.export_flat_vectors.connect(my_export_flat_vectors)
    >>> my_data1 = dpf.()
    >>> op.inputs.data1.connect(my_data1)
    >>> my_data2 = dpf.()
    >>> op.inputs.data2.connect(my_data2)
    """

    def __init__(self, op: Operator):
        super().__init__(serialize_to_hdf5._spec().inputs, op)
        self._file_path = Input(serialize_to_hdf5._spec().input_pin(0), 0, op, -1)
        self._inputs.append(self._file_path)
        self._export_floats = Input(serialize_to_hdf5._spec().input_pin(1), 1, op, -1)
        self._inputs.append(self._export_floats)
        self._export_flat_vectors = Input(
            serialize_to_hdf5._spec().input_pin(2), 2, op, -1
        )
        self._inputs.append(self._export_flat_vectors)
        self._data1 = Input(serialize_to_hdf5._spec().input_pin(3), 3, op, 0)
        self._inputs.append(self._data1)
        self._data2 = Input(serialize_to_hdf5._spec().input_pin(4), 4, op, 1)
        self._inputs.append(self._data2)

    @property
    def file_path(self):
        """Allows to connect file_path input to the operator.

        Output file path with .h5 extension

        Parameters
        ----------
        my_file_path : str

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.serialization.serialize_to_hdf5()
        >>> op.inputs.file_path.connect(my_file_path)
        >>> # or
        >>> op.inputs.file_path(my_file_path)
        """
        return self._file_path

    @property
    def export_floats(self):
        """Allows to connect export_floats input to the operator.

        Converts double to float to reduce file size
        (default is true)

        Parameters
        ----------
        my_export_floats : bool

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.serialization.serialize_to_hdf5()
        >>> op.inputs.export_floats.connect(my_export_floats)
        >>> # or
        >>> op.inputs.export_floats(my_export_floats)
        """
        return self._export_floats

    @property
    def export_flat_vectors(self):
        """Allows to connect export_flat_vectors input to the operator.

        If true, vectors and matrices data are
        exported flat (x1,y1,z1,x2,y2,z2..)
        (default is false)

        Parameters
        ----------
        my_export_flat_vectors : bool

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.serialization.serialize_to_hdf5()
        >>> op.inputs.export_flat_vectors.connect(my_export_flat_vectors)
        >>> # or
        >>> op.inputs.export_flat_vectors(my_export_flat_vectors)
        """
        return self._export_flat_vectors

    @property
    def data1(self):
        """Allows to connect data1 input to the operator.

        Only the data set explicitly to export is
        exported

        Parameters
        ----------
        my_data1 :

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.serialization.serialize_to_hdf5()
        >>> op.inputs.data1.connect(my_data1)
        >>> # or
        >>> op.inputs.data1(my_data1)
        """
        return self._data1

    @property
    def data2(self):
        """Allows to connect data2 input to the operator.

        Only the data set explicitly to export is
        exported

        Parameters
        ----------
        my_data2 :

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.serialization.serialize_to_hdf5()
        >>> op.inputs.data2.connect(my_data2)
        >>> # or
        >>> op.inputs.data2(my_data2)
        """
        return self._data2


class OutputsSerializeToHdf5(_Outputs):
    """Intermediate class used to get outputs from
    serialize_to_hdf5 operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.serialization.serialize_to_hdf5()
    >>> # Connect inputs : op.inputs. ...
    """

    def __init__(self, op: Operator):
        super().__init__(serialize_to_hdf5._spec().outputs, op)



"""
connectivity_ids
================
Autogenerated DPF operator classes.
"""
from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification


class connectivity_ids(Operator):
    """Returns the ordered node ids corresponding to the element ids scoping
    in input. For each element the node ids are its connectivity.

    Parameters
    ----------
    mesh_scoping : Scoping
        Elemental scoping
    mesh : MeshedRegion, optional
        The support of the scoping is expected if
        there is no mesh in input
    take_mid_nodes : bool, optional
        Default is true


    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.scoping.connectivity_ids()

    >>> # Make input connections
    >>> my_mesh_scoping = dpf.Scoping()
    >>> op.inputs.mesh_scoping.connect(my_mesh_scoping)
    >>> my_mesh = dpf.MeshedRegion()
    >>> op.inputs.mesh.connect(my_mesh)
    >>> my_take_mid_nodes = bool()
    >>> op.inputs.take_mid_nodes.connect(my_take_mid_nodes)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.scoping.connectivity_ids(
    ...     mesh_scoping=my_mesh_scoping,
    ...     mesh=my_mesh,
    ...     take_mid_nodes=my_take_mid_nodes,
    ... )

    >>> # Get output data
    >>> result_mesh_scoping = op.outputs.mesh_scoping()
    >>> result_elemental_scoping = op.outputs.elemental_scoping()
    """

    def __init__(
        self,
        mesh_scoping=None,
        mesh=None,
        take_mid_nodes=None,
        config=None,
        server=None,
    ):
        super().__init__(name="scoping::connectivity_ids", config=config, server=server)
        self._inputs = InputsConnectivityIds(self)
        self._outputs = OutputsConnectivityIds(self)
        if mesh_scoping is not None:
            self.inputs.mesh_scoping.connect(mesh_scoping)
        if mesh is not None:
            self.inputs.mesh.connect(mesh)
        if take_mid_nodes is not None:
            self.inputs.take_mid_nodes.connect(take_mid_nodes)

    @staticmethod
    def _spec():
        description = """Returns the ordered node ids corresponding to the element ids scoping
            in input. For each element the node ids are its
            connectivity."""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                1: PinSpecification(
                    name="mesh_scoping",
                    type_names=["scoping"],
                    optional=False,
                    document="""Elemental scoping""",
                ),
                7: PinSpecification(
                    name="mesh",
                    type_names=["abstract_meshed_region"],
                    optional=True,
                    document="""The support of the scoping is expected if
        there is no mesh in input""",
                ),
                10: PinSpecification(
                    name="take_mid_nodes",
                    type_names=["bool"],
                    optional=True,
                    document="""Default is true""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="mesh_scoping",
                    type_names=["scoping"],
                    optional=False,
                    document="""""",
                ),
                1: PinSpecification(
                    name="elemental_scoping",
                    type_names=["scoping"],
                    optional=False,
                    document="""Same as the input scoping but with ids
        duplicated to have the same size as
        nodal output scoping""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server=None):
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server : server.DPFServer, optional
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.
        """
        return Operator.default_config(name="scoping::connectivity_ids", server=server)

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsConnectivityIds
        """
        return super().inputs

    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs : OutputsConnectivityIds
        """
        return super().outputs


class InputsConnectivityIds(_Inputs):
    """Intermediate class used to connect user inputs to
    connectivity_ids operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.scoping.connectivity_ids()
    >>> my_mesh_scoping = dpf.Scoping()
    >>> op.inputs.mesh_scoping.connect(my_mesh_scoping)
    >>> my_mesh = dpf.MeshedRegion()
    >>> op.inputs.mesh.connect(my_mesh)
    >>> my_take_mid_nodes = bool()
    >>> op.inputs.take_mid_nodes.connect(my_take_mid_nodes)
    """

    def __init__(self, op: Operator):
        super().__init__(connectivity_ids._spec().inputs, op)
        self._mesh_scoping = Input(connectivity_ids._spec().input_pin(1), 1, op, -1)
        self._inputs.append(self._mesh_scoping)
        self._mesh = Input(connectivity_ids._spec().input_pin(7), 7, op, -1)
        self._inputs.append(self._mesh)
        self._take_mid_nodes = Input(connectivity_ids._spec().input_pin(10), 10, op, -1)
        self._inputs.append(self._take_mid_nodes)

    @property
    def mesh_scoping(self):
        """Allows to connect mesh_scoping input to the operator.

        Elemental scoping

        Parameters
        ----------
        my_mesh_scoping : Scoping

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.scoping.connectivity_ids()
        >>> op.inputs.mesh_scoping.connect(my_mesh_scoping)
        >>> # or
        >>> op.inputs.mesh_scoping(my_mesh_scoping)
        """
        return self._mesh_scoping

    @property
    def mesh(self):
        """Allows to connect mesh input to the operator.

        The support of the scoping is expected if
        there is no mesh in input

        Parameters
        ----------
        my_mesh : MeshedRegion

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.scoping.connectivity_ids()
        >>> op.inputs.mesh.connect(my_mesh)
        >>> # or
        >>> op.inputs.mesh(my_mesh)
        """
        return self._mesh

    @property
    def take_mid_nodes(self):
        """Allows to connect take_mid_nodes input to the operator.

        Default is true

        Parameters
        ----------
        my_take_mid_nodes : bool

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.scoping.connectivity_ids()
        >>> op.inputs.take_mid_nodes.connect(my_take_mid_nodes)
        >>> # or
        >>> op.inputs.take_mid_nodes(my_take_mid_nodes)
        """
        return self._take_mid_nodes


class OutputsConnectivityIds(_Outputs):
    """Intermediate class used to get outputs from
    connectivity_ids operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.scoping.connectivity_ids()
    >>> # Connect inputs : op.inputs. ...
    >>> result_mesh_scoping = op.outputs.mesh_scoping()
    >>> result_elemental_scoping = op.outputs.elemental_scoping()
    """

    def __init__(self, op: Operator):
        super().__init__(connectivity_ids._spec().outputs, op)
        self._mesh_scoping = Output(connectivity_ids._spec().output_pin(0), 0, op)
        self._outputs.append(self._mesh_scoping)
        self._elemental_scoping = Output(connectivity_ids._spec().output_pin(1), 1, op)
        self._outputs.append(self._elemental_scoping)

    @property
    def mesh_scoping(self):
        """Allows to get mesh_scoping output of the operator

        Returns
        ----------
        my_mesh_scoping : Scoping

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.scoping.connectivity_ids()
        >>> # Connect inputs : op.inputs. ...
        >>> result_mesh_scoping = op.outputs.mesh_scoping()
        """  # noqa: E501
        return self._mesh_scoping

    @property
    def elemental_scoping(self):
        """Allows to get elemental_scoping output of the operator

        Returns
        ----------
        my_elemental_scoping : Scoping

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.scoping.connectivity_ids()
        >>> # Connect inputs : op.inputs. ...
        >>> result_elemental_scoping = op.outputs.elemental_scoping()
        """  # noqa: E501
        return self._elemental_scoping



"""
modal_superposition
===================
Autogenerated DPF operator classes.
"""
from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification


class modal_superposition(Operator):
    """Computes the solution in the time/frequency space from a modal
    solution by multiplying a modal basis (in 0) by the solution in
    this modal space (coefficients for each mode for each
    time/frequency) (in 1).

    Parameters
    ----------
    modal_basis : FieldsContainer
        One field by mode with each field
        representing a mode shape on nodes or
        elements.
    solution_in_modal_space : FieldsContainer
        One field by time/frequency with each field
        having a ponderating coefficient for
        each mode of the modal_basis pin.
    time_scoping : Scoping, optional
        Compute the result on a subset of the time
        frequency domain defined in the
        solution_in_modal_space fields
        container.
    mesh_scoping : Scoping or ScopingsContainer, optional
        Compute the result on a subset of the space
        domain defined in the modal_basis
        fields container.


    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.math.modal_superposition()

    >>> # Make input connections
    >>> my_modal_basis = dpf.FieldsContainer()
    >>> op.inputs.modal_basis.connect(my_modal_basis)
    >>> my_solution_in_modal_space = dpf.FieldsContainer()
    >>> op.inputs.solution_in_modal_space.connect(my_solution_in_modal_space)
    >>> my_time_scoping = dpf.Scoping()
    >>> op.inputs.time_scoping.connect(my_time_scoping)
    >>> my_mesh_scoping = dpf.Scoping()
    >>> op.inputs.mesh_scoping.connect(my_mesh_scoping)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.math.modal_superposition(
    ...     modal_basis=my_modal_basis,
    ...     solution_in_modal_space=my_solution_in_modal_space,
    ...     time_scoping=my_time_scoping,
    ...     mesh_scoping=my_mesh_scoping,
    ... )

    >>> # Get output data
    >>> result_fields_container = op.outputs.fields_container()
    """

    def __init__(
        self,
        modal_basis=None,
        solution_in_modal_space=None,
        time_scoping=None,
        mesh_scoping=None,
        config=None,
        server=None,
    ):
        super().__init__(
            name="expansion::modal_superposition", config=config, server=server
        )
        self._inputs = InputsModalSuperposition(self)
        self._outputs = OutputsModalSuperposition(self)
        if modal_basis is not None:
            self.inputs.modal_basis.connect(modal_basis)
        if solution_in_modal_space is not None:
            self.inputs.solution_in_modal_space.connect(solution_in_modal_space)
        if time_scoping is not None:
            self.inputs.time_scoping.connect(time_scoping)
        if mesh_scoping is not None:
            self.inputs.mesh_scoping.connect(mesh_scoping)

    @staticmethod
    def _spec():
        description = """Computes the solution in the time/frequency space from a modal
            solution by multiplying a modal basis (in 0) by the
            solution in this modal space (coefficients for each mode
            for each time/frequency) (in 1)."""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                0: PinSpecification(
                    name="modal_basis",
                    type_names=["fields_container"],
                    optional=False,
                    document="""One field by mode with each field
        representing a mode shape on nodes or
        elements.""",
                ),
                1: PinSpecification(
                    name="solution_in_modal_space",
                    type_names=["fields_container"],
                    optional=False,
                    document="""One field by time/frequency with each field
        having a ponderating coefficient for
        each mode of the modal_basis pin.""",
                ),
                3: PinSpecification(
                    name="time_scoping",
                    type_names=["scoping", "vector<int32>"],
                    optional=True,
                    document="""Compute the result on a subset of the time
        frequency domain defined in the
        solution_in_modal_space fields
        container.""",
                ),
                4: PinSpecification(
                    name="mesh_scoping",
                    type_names=["scoping", "scopings_container"],
                    optional=True,
                    document="""Compute the result on a subset of the space
        domain defined in the modal_basis
        fields container.""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="fields_container",
                    type_names=["fields_container"],
                    optional=False,
                    document="""""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server=None):
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server : server.DPFServer, optional
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.
        """
        return Operator.default_config(
            name="expansion::modal_superposition", server=server
        )

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsModalSuperposition
        """
        return super().inputs

    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs : OutputsModalSuperposition
        """
        return super().outputs


class InputsModalSuperposition(_Inputs):
    """Intermediate class used to connect user inputs to
    modal_superposition operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.math.modal_superposition()
    >>> my_modal_basis = dpf.FieldsContainer()
    >>> op.inputs.modal_basis.connect(my_modal_basis)
    >>> my_solution_in_modal_space = dpf.FieldsContainer()
    >>> op.inputs.solution_in_modal_space.connect(my_solution_in_modal_space)
    >>> my_time_scoping = dpf.Scoping()
    >>> op.inputs.time_scoping.connect(my_time_scoping)
    >>> my_mesh_scoping = dpf.Scoping()
    >>> op.inputs.mesh_scoping.connect(my_mesh_scoping)
    """

    def __init__(self, op: Operator):
        super().__init__(modal_superposition._spec().inputs, op)
        self._modal_basis = Input(modal_superposition._spec().input_pin(0), 0, op, -1)
        self._inputs.append(self._modal_basis)
        self._solution_in_modal_space = Input(
            modal_superposition._spec().input_pin(1), 1, op, -1
        )
        self._inputs.append(self._solution_in_modal_space)
        self._time_scoping = Input(modal_superposition._spec().input_pin(3), 3, op, -1)
        self._inputs.append(self._time_scoping)
        self._mesh_scoping = Input(modal_superposition._spec().input_pin(4), 4, op, -1)
        self._inputs.append(self._mesh_scoping)

    @property
    def modal_basis(self):
        """Allows to connect modal_basis input to the operator.

        One field by mode with each field
        representing a mode shape on nodes or
        elements.

        Parameters
        ----------
        my_modal_basis : FieldsContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.math.modal_superposition()
        >>> op.inputs.modal_basis.connect(my_modal_basis)
        >>> # or
        >>> op.inputs.modal_basis(my_modal_basis)
        """
        return self._modal_basis

    @property
    def solution_in_modal_space(self):
        """Allows to connect solution_in_modal_space input to the operator.

        One field by time/frequency with each field
        having a ponderating coefficient for
        each mode of the modal_basis pin.

        Parameters
        ----------
        my_solution_in_modal_space : FieldsContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.math.modal_superposition()
        >>> op.inputs.solution_in_modal_space.connect(my_solution_in_modal_space)
        >>> # or
        >>> op.inputs.solution_in_modal_space(my_solution_in_modal_space)
        """
        return self._solution_in_modal_space

    @property
    def time_scoping(self):
        """Allows to connect time_scoping input to the operator.

        Compute the result on a subset of the time
        frequency domain defined in the
        solution_in_modal_space fields
        container.

        Parameters
        ----------
        my_time_scoping : Scoping

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.math.modal_superposition()
        >>> op.inputs.time_scoping.connect(my_time_scoping)
        >>> # or
        >>> op.inputs.time_scoping(my_time_scoping)
        """
        return self._time_scoping

    @property
    def mesh_scoping(self):
        """Allows to connect mesh_scoping input to the operator.

        Compute the result on a subset of the space
        domain defined in the modal_basis
        fields container.

        Parameters
        ----------
        my_mesh_scoping : Scoping or ScopingsContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.math.modal_superposition()
        >>> op.inputs.mesh_scoping.connect(my_mesh_scoping)
        >>> # or
        >>> op.inputs.mesh_scoping(my_mesh_scoping)
        """
        return self._mesh_scoping


class OutputsModalSuperposition(_Outputs):
    """Intermediate class used to get outputs from
    modal_superposition operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.math.modal_superposition()
    >>> # Connect inputs : op.inputs. ...
    >>> result_fields_container = op.outputs.fields_container()
    """

    def __init__(self, op: Operator):
        super().__init__(modal_superposition._spec().outputs, op)
        self._fields_container = Output(
            modal_superposition._spec().output_pin(0), 0, op
        )
        self._outputs.append(self._fields_container)

    @property
    def fields_container(self):
        """Allows to get fields_container output of the operator

        Returns
        ----------
        my_fields_container : FieldsContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.math.modal_superposition()
        >>> # Connect inputs : op.inputs. ...
        >>> result_fields_container = op.outputs.fields_container()
        """  # noqa: E501
        return self._fields_container



"""
AvailableResult
===============
"""

from typing import List
from warnings import warn
from ansys.dpf.core.common import _remove_spaces, _make_as_function_name, natures
from enum import Enum, unique


@unique
class Homogeneity(Enum):
    acceleration = 0
    angle = 1
    angular_velocity = 2
    surface = 3
    capacitance = 4
    electric_charge = 5
    electric_charge_density = 6
    conductivity = 7
    current = 9
    density = 10
    displacement = 11
    electric_conductivity = 12
    electric_field = 13
    electric_flux_density = 14
    electric_resistivity = 15
    energy = 16
    film_coefficient = 17
    force = 18
    force_intensity = 19
    frequency = 20
    heat_flux = 21
    heat_generation = 22
    heat_rate = 23
    inductance = 24
    inverse_stress = 25
    length = 26
    magnetic_field_intensity = 27
    magnetic_flux = 28
    magnetic_flux_density = 29
    mass = 30
    moment = 31
    moment_inertia = 32
    permeability = 33
    permittivity = 34
    poisson = 35
    power = 36
    pressure = 37
    relative_permeability = 38
    relative_permittivity = 39
    section_modulus = 40
    specific_heat = 41
    specific_weight = 42
    shear_strain = 43
    stiffness = 44
    strain = 45
    stress = 46
    strength = 47
    thermal_expansion = 48
    temperature = 49
    time = 50
    velocity = 51
    voltage = 52
    volume = 53
    moment_inertia_mass = 55
    stress_intensity_factor = 92
    thermal_gradient = 95
    resistance = 1000
    dimensionless = 117
    dynamic_viscosity = 118
    dissipation_rate = 119
    momentum = 120
    volume_flow_rate = 121
    mass_flow_rate = 122
    specific_energy = 123
    specific_entropy = 124


class AvailableResult:
    """Represents a result that can be requested via an operator.

    Parameters
    ----------
    availableresult : available_result_pb2.AvailableResult message.

    Examples
    --------
    Explore an available result from the model.

    >>> from ansys.dpf import core as dpf
    >>> from ansys.dpf.core import examples
    >>> transient = examples.download_transient_result()
    >>> model = dpf.Model(transient)
    >>> result_info = model.metadata.result_info
    >>> res = result_info.available_results[0]
    >>> res.name
    'displacement'
    >>> res.homogeneity
    'length'
    >>> res.dimensionality
    'vector'

    Create the operator of the given available result.

    >>> disp = model.results.displacement()

    """

    def __init__(self, availableresult):
        """Initialize the AvailableResult with an availableResult message."""
        self._name = availableresult.name
        self._physics_name = availableresult.physicsname
        self._dimensionality = availableresult.dimensionality
        self._homogeneity = availableresult.homogeneity
        self._unit = availableresult.unit
        self._n_comp = availableresult.ncomp
        self._properties = {
            "scripting_name": availableresult.properties["scripting_name"],
            "location": availableresult.properties["loc_name"],
        }
        self._sub_res = availableresult.sub_res
        self._qualifiers = availableresult.qualifiers
        self._qualifier_labels = availableresult.qualifier_labels

    def __str__(self):
        txt = (
            "DPF Result\n----------\n"
            + self.name
            + "\n"
            + 'Operator name: "%s"\n' % self.operator_name
            + "Number of components: %d\n" % self.n_components
            + "Dimensionality: %s\n" % self.dimensionality
            + "Homogeneity: %s\n" % self.homogeneity
        )
        if self.unit:
            txt += "Units: %s\n" % self.unit
        if self.native_location:
            txt += "Location: %s\n" % self.native_location
        if self.qualifiers:
            txt += "Available qualifier labels:\n"
            for label in self.qualifier_labels:
                txt += f"  - {label}: {', '.join(map(str, self.qualifier_labels[label]))}\n"
            txt += "Available qualifier combinations:\n"
            for qualifier in self.qualifiers:
                txt += f"  {qualifier.__dict__()}\n"
        return txt

    def __repr__(self):
        return f"AvailableResult<name={self.name}>"

    @property
    def name(self):
        """Result operator."""
        if hasattr(self, "properties") and "scripting_name" in self._properties.keys():
            name = self.properties["scripting_name"]
        elif self.operator_name in _result_properties:
            name = _result_properties[self.operator_name]["scripting_name"]
        else:
            name = _remove_spaces(self._physics_name)
        return _make_as_function_name(name)

    @property
    def n_components(self):
        """Number of components of the result."""
        return self._n_comp

    @property
    def dimensionality(self):
        """Dimensionality nature of the result, such as a vector, scalar, or tensor."""
        return natures(self._dimensionality).name

    @property
    def homogeneity(self):
        """Homogeneity of the result."""
        try:
            # homogeneity = self._homogeneity
            # if homogeneity == 117:
            #     return Homogeneity(Homogeneity.DIMENSIONLESS).name
            return Homogeneity(self._homogeneity).name
        except ValueError as exception:
            warn(str(exception))
            return ""

    @property
    def unit(self):
        """Unit of the result."""
        return self._unit.lower()

    @property
    def operator_name(self):
        """Name of the corresponding operator."""
        return self._name

    @property
    def sub_results(self):
        """List of the subresult."""
        rep_sub_res = self._sub_res
        list_of_rep = []
        for sub_res_name in rep_sub_res.keys():
            sub_res = rep_sub_res[sub_res_name]
            try:
                int(sub_res_name)
                rep_sub = {
                    "name": "principal" + sub_res_name,
                    "operator name": sub_res[0],
                    "description": sub_res[1],
                }
            except:
                rep_sub = {
                    "name": sub_res_name,
                    "operator name": sub_res[0],
                    "description": sub_res[1],
                }
            list_of_rep.append(rep_sub)
        return list_of_rep

    @property
    def native_location(self):
        """Native location of the result."""
        if hasattr(self, "_properties") and "location" in self._properties.keys():
            return self._properties["location"]
        if self.operator_name in _result_properties:
            return _result_properties[self.operator_name]["location"]

    @property
    def native_scoping_location(self):
        """Native scoping location of the result."""
        loc = self.native_location
        if loc == "ElementalNodal":
            return "Elemental"
        else:
            return loc

    @property
    def physical_name(self) -> str:
        """Name of the result with spaces"""
        return self._physics_name

    @property
    def qualifiers(self) -> list:
        """Returns the list of qualifiers (equivalent to label spaces)
        available for a given Result. These qualifiers can then be used to request the result
        on specified locations/properties.
        """
        return self._qualifiers

    @property
    def qualifier_labels(self) -> dict:
        """Returns a dictionary of available labels for each available qualifier."""
        return self._qualifier_labels

    @property
    def qualifier_combinations(self) -> List[dict]:
        """Returns the list of valid qualifier combinations for this result.

        Each combination is a dictionary which can be used for a result request.
        """
        return [q.__dict__() for q in self.qualifiers]


_result_properties = {
    "S": {"location": "ElementalNodal", "scripting_name": "stress"},
    "ENF": {"location": "ElementalNodal", "scripting_name": "element_nodal_forces"},
    "EPEL": {"location": "ElementalNodal", "scripting_name": "elastic_strain"},
    "EPPL": {"location": "ElementalNodal", "scripting_name": "plastic_strain"},
    "ECR": {"location": "ElementalNodal", "scripting_name": "creep_strain"},
    "BFE": {"location": "ElementalNodal", "scripting_name": "structural_temperature"},
    "ETH": {"location": "ElementalNodal", "scripting_name": "thermal_strain"},
    "ETH_SWL": {"location": "ElementalNodal", "scripting_name": "swelling_strains"},
    "ENG_VOL": {"location": "Elemental", "scripting_name": "elemental_volume"},
    "ENG_SE": {"location": "Elemental", "scripting_name": "stiffness_matrix_energy"},
    "ENG_AHO": {
        "location": "Elemental",
        "scripting_name": "artificial_hourglass_energy",
    },
    "ENG_KE": {"location": "Elemental", "scripting_name": "kinetic_energy"},
    "ENG_CO": {"location": "Elemental", "scripting_name": "co_energy"},
    "ENG_INC": {"location": "Elemental", "scripting_name": "incremental_energy"},
    "ENG_TH": {"location": "Elemental", "scripting_name": "thermal_dissipation_energy"},
    "U": {"location": "Nodal", "scripting_name": "displacement"},
    "V": {"location": "Nodal", "scripting_name": "velocity"},
    "A": {"location": "Nodal", "scripting_name": "acceleration"},
    "RF": {"location": "Nodal", "scripting_name": "reaction_force"},
    "F": {"location": "Nodal", "scripting_name": "nodal_force"},
    "M": {"location": "Nodal", "scripting_name": "nodal_moment"},
    "TEMP": {"location": "Nodal", "scripting_name": "temperature"},
    "EF": {"location": "ElementalNodal", "scripting_name": "electric_field"},
    "VOLT": {"location": "Nodal", "scripting_name": "electric_potential"},
    "TF": {"location": "ElementalNodal", "scripting_name": "heat_flux"},
    "UTOT": {"location": "Nodal", "scripting_name": "raw_displacement"},
    "RFTOT": {"location": "Nodal", "scripting_name": "raw_reaction_force"},
}


def available_result_from_name(name) -> AvailableResult:
    for key, item in _result_properties.items():
        if item["scripting_name"] == name:
            from types import SimpleNamespace

            availableresult = SimpleNamespace(
                name=key,
                physicsname=name,
                ncomp=None,
                dimensionality=None,
                homogeneity=None,
                unit=None,
                sub_res={},
                properties={"loc_name": item["location"], "scripting_name": name},
                qualifiers=[],
                qualifier_labels={},
            )

            return AvailableResult(availableresult)



"""
extract_sub_sc
==============
Autogenerated DPF operator classes.
"""
from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification


class extract_sub_sc(Operator):
    """Creates a new scopings container with all the scopings corresponding
    to the label space in input 1.

    Parameters
    ----------
    scopings_container : ScopingsContainer
        Scopings_container
    label_space : dict
        Label_space


    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.utility.extract_sub_sc()

    >>> # Make input connections
    >>> my_scopings_container = dpf.ScopingsContainer()
    >>> op.inputs.scopings_container.connect(my_scopings_container)
    >>> my_label_space = dict()
    >>> op.inputs.label_space.connect(my_label_space)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.utility.extract_sub_sc(
    ...     scopings_container=my_scopings_container,
    ...     label_space=my_label_space,
    ... )

    >>> # Get output data
    >>> result_scopings_container = op.outputs.scopings_container()
    """

    def __init__(
        self, scopings_container=None, label_space=None, config=None, server=None
    ):
        super().__init__(name="extract_sub_sc", config=config, server=server)
        self._inputs = InputsExtractSubSc(self)
        self._outputs = OutputsExtractSubSc(self)
        if scopings_container is not None:
            self.inputs.scopings_container.connect(scopings_container)
        if label_space is not None:
            self.inputs.label_space.connect(label_space)

    @staticmethod
    def _spec():
        description = """Creates a new scopings container with all the scopings corresponding
            to the label space in input 1."""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                0: PinSpecification(
                    name="scopings_container",
                    type_names=["scopings_container"],
                    optional=False,
                    document="""Scopings_container""",
                ),
                1: PinSpecification(
                    name="label_space",
                    type_names=["label_space"],
                    optional=False,
                    document="""Label_space""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="scopings_container",
                    type_names=["scopings_container"],
                    optional=False,
                    document="""Scopings_container""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server=None):
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server : server.DPFServer, optional
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.
        """
        return Operator.default_config(name="extract_sub_sc", server=server)

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsExtractSubSc
        """
        return super().inputs

    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs : OutputsExtractSubSc
        """
        return super().outputs


class InputsExtractSubSc(_Inputs):
    """Intermediate class used to connect user inputs to
    extract_sub_sc operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.utility.extract_sub_sc()
    >>> my_scopings_container = dpf.ScopingsContainer()
    >>> op.inputs.scopings_container.connect(my_scopings_container)
    >>> my_label_space = dict()
    >>> op.inputs.label_space.connect(my_label_space)
    """

    def __init__(self, op: Operator):
        super().__init__(extract_sub_sc._spec().inputs, op)
        self._scopings_container = Input(extract_sub_sc._spec().input_pin(0), 0, op, -1)
        self._inputs.append(self._scopings_container)
        self._label_space = Input(extract_sub_sc._spec().input_pin(1), 1, op, -1)
        self._inputs.append(self._label_space)

    @property
    def scopings_container(self):
        """Allows to connect scopings_container input to the operator.

        Scopings_container

        Parameters
        ----------
        my_scopings_container : ScopingsContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.extract_sub_sc()
        >>> op.inputs.scopings_container.connect(my_scopings_container)
        >>> # or
        >>> op.inputs.scopings_container(my_scopings_container)
        """
        return self._scopings_container

    @property
    def label_space(self):
        """Allows to connect label_space input to the operator.

        Label_space

        Parameters
        ----------
        my_label_space : dict

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.extract_sub_sc()
        >>> op.inputs.label_space.connect(my_label_space)
        >>> # or
        >>> op.inputs.label_space(my_label_space)
        """
        return self._label_space


class OutputsExtractSubSc(_Outputs):
    """Intermediate class used to get outputs from
    extract_sub_sc operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.utility.extract_sub_sc()
    >>> # Connect inputs : op.inputs. ...
    >>> result_scopings_container = op.outputs.scopings_container()
    """

    def __init__(self, op: Operator):
        super().__init__(extract_sub_sc._spec().outputs, op)
        self._scopings_container = Output(extract_sub_sc._spec().output_pin(0), 0, op)
        self._outputs.append(self._scopings_container)

    @property
    def scopings_container(self):
        """Allows to get scopings_container output of the operator

        Returns
        ----------
        my_scopings_container : ScopingsContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.extract_sub_sc()
        >>> # Connect inputs : op.inputs. ...
        >>> result_scopings_container = op.outputs.scopings_container()
        """  # noqa: E501
        return self._scopings_container



"""
producer_consumer_for_each
==========================
Autogenerated DPF operator classes.
"""
from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification


class producer_consumer_for_each(Operator):
    """Allows to write a loop over operators by connecting data to iterate
    and by requesting the incrementally output merged.The chain of
    Operators are split into a first part: the producers and a second
    part: the consumers. These 2 parts will run asynchronously on 2
    threads.

    Parameters
    ----------
    producer_consumer_iterableq :
        The result of the
        make_producer_consumer_for_each_itera
        tor operator.
    forward1 :
        Output of the last operators of the workflow
    forward2 :
        Output of the last operators of the workflow


    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.utility.producer_consumer_for_each()

    >>> # Make input connections
    >>> my_producer_consumer_iterableq = dpf.()
    >>> op.inputs.producer_consumer_iterableq.connect(my_producer_consumer_iterableq)
    >>> my_forward1 = dpf.()
    >>> op.inputs.forward1.connect(my_forward1)
    >>> my_forward2 = dpf.()
    >>> op.inputs.forward2.connect(my_forward2)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.utility.producer_consumer_for_each(
    ...     producer_consumer_iterableq=my_producer_consumer_iterableq,
    ...     forward1=my_forward1,
    ...     forward2=my_forward2,
    ... )

    >>> # Get output data
    >>> result_output1 = op.outputs.output1()
    >>> result_output2 = op.outputs.output2()
    """

    def __init__(
        self,
        producer_consumer_iterableq=None,
        forward1=None,
        forward2=None,
        config=None,
        server=None,
    ):
        super().__init__(
            name="producer_consumer_for_each", config=config, server=server
        )
        self._inputs = InputsProducerConsumerForEach(self)
        self._outputs = OutputsProducerConsumerForEach(self)
        if producer_consumer_iterableq is not None:
            self.inputs.producer_consumer_iterableq.connect(producer_consumer_iterableq)
        if forward1 is not None:
            self.inputs.forward1.connect(forward1)
        if forward2 is not None:
            self.inputs.forward2.connect(forward2)

    @staticmethod
    def _spec():
        description = """Allows to write a loop over operators by connecting data to iterate
            and by requesting the incrementally output merged.The
            chain of Operators are split into a first part: the
            producers and a second part: the consumers. These 2 parts
            will run asynchronously on 2 threads."""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                0: PinSpecification(
                    name="producer_consumer_iterableq",
                    type_names=["any"],
                    optional=False,
                    document="""The result of the
        make_producer_consumer_for_each_itera
        tor operator.""",
                ),
                3: PinSpecification(
                    name="forward",
                    type_names=["any"],
                    optional=False,
                    document="""Output of the last operators of the workflow""",
                ),
                4: PinSpecification(
                    name="forward",
                    type_names=["any"],
                    optional=False,
                    document="""Output of the last operators of the workflow""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="output1",
                    optional=False,
                    document="""""",
                ),
                1: PinSpecification(
                    name="output2",
                    optional=False,
                    document="""""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server=None):
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server : server.DPFServer, optional
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.
        """
        return Operator.default_config(name="producer_consumer_for_each", server=server)

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsProducerConsumerForEach
        """
        return super().inputs

    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs : OutputsProducerConsumerForEach
        """
        return super().outputs


class InputsProducerConsumerForEach(_Inputs):
    """Intermediate class used to connect user inputs to
    producer_consumer_for_each operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.utility.producer_consumer_for_each()
    >>> my_producer_consumer_iterableq = dpf.()
    >>> op.inputs.producer_consumer_iterableq.connect(my_producer_consumer_iterableq)
    >>> my_forward1 = dpf.()
    >>> op.inputs.forward1.connect(my_forward1)
    >>> my_forward2 = dpf.()
    >>> op.inputs.forward2.connect(my_forward2)
    """

    def __init__(self, op: Operator):
        super().__init__(producer_consumer_for_each._spec().inputs, op)
        self._producer_consumer_iterableq = Input(
            producer_consumer_for_each._spec().input_pin(0), 0, op, -1
        )
        self._inputs.append(self._producer_consumer_iterableq)
        self._forward1 = Input(
            producer_consumer_for_each._spec().input_pin(3), 3, op, 0
        )
        self._inputs.append(self._forward1)
        self._forward2 = Input(
            producer_consumer_for_each._spec().input_pin(4), 4, op, 1
        )
        self._inputs.append(self._forward2)

    @property
    def producer_consumer_iterableq(self):
        """Allows to connect producer_consumer_iterableq input to the operator.

        The result of the
        make_producer_consumer_for_each_itera
        tor operator.

        Parameters
        ----------
        my_producer_consumer_iterableq :

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.producer_consumer_for_each()
        >>> op.inputs.producer_consumer_iterableq.connect(my_producer_consumer_iterableq)
        >>> # or
        >>> op.inputs.producer_consumer_iterableq(my_producer_consumer_iterableq)
        """
        return self._producer_consumer_iterableq

    @property
    def forward1(self):
        """Allows to connect forward1 input to the operator.

        Output of the last operators of the workflow

        Parameters
        ----------
        my_forward1 :

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.producer_consumer_for_each()
        >>> op.inputs.forward1.connect(my_forward1)
        >>> # or
        >>> op.inputs.forward1(my_forward1)
        """
        return self._forward1

    @property
    def forward2(self):
        """Allows to connect forward2 input to the operator.

        Output of the last operators of the workflow

        Parameters
        ----------
        my_forward2 :

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.producer_consumer_for_each()
        >>> op.inputs.forward2.connect(my_forward2)
        >>> # or
        >>> op.inputs.forward2(my_forward2)
        """
        return self._forward2


class OutputsProducerConsumerForEach(_Outputs):
    """Intermediate class used to get outputs from
    producer_consumer_for_each operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.utility.producer_consumer_for_each()
    >>> # Connect inputs : op.inputs. ...
    >>> result_output1 = op.outputs.output1()
    >>> result_output2 = op.outputs.output2()
    """

    def __init__(self, op: Operator):
        super().__init__(producer_consumer_for_each._spec().outputs, op)
        self._output1 = Output(producer_consumer_for_each._spec().output_pin(0), 0, op)
        self._outputs.append(self._output1)
        self._output2 = Output(producer_consumer_for_each._spec().output_pin(1), 1, op)
        self._outputs.append(self._output2)

    @property
    def output1(self):
        """Allows to get output1 output of the operator

        Returns
        ----------
        my_output1 :

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.producer_consumer_for_each()
        >>> # Connect inputs : op.inputs. ...
        >>> result_output1 = op.outputs.output1()
        """  # noqa: E501
        return self._output1

    @property
    def output2(self):
        """Allows to get output2 output of the operator

        Returns
        ----------
        my_output2 :

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.producer_consumer_for_each()
        >>> # Connect inputs : op.inputs. ...
        >>> result_output2 = op.outputs.output2()
        """  # noqa: E501
        return self._output2



"""
mapdl_material_properties
=========================
Autogenerated DPF operator classes.
"""
from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification


class mapdl_material_properties(Operator):
    """Read the values of the properties of a material for a given materials
    property field (property field that contains materials information
    for each element of a mesh).It returns a fields container
    containing a field for each material property, with only one value
    per material. The following keys can be used: Young's modulus
    (keys: EX, EY, EZ), Poisson's ratio (keys: NUXY, NUYZ, NUXZ),
    Shear Modulus (keys: GXY, GYZ, GXZ), Coefficient of Thermal
    Expansion (keys: ALPX, ALPY, ALPZ), Volumic Mass (key: DENS),
    second Lame's coefficient (key: MU), Damping coefficient (key:
    DAMP), thermal Conductivity (keys: KXX, KYY, KZZ), Resistivity
    (keys: RSVX, RSVY, RSVZ), Specific heat in constant volume (key:
    C), Film coefficient (key: HF), Viscosity (key: VISC), Emissivity
    (key: EMIS).

    Parameters
    ----------
    properties_name : str
    materials : PropertyField
        Property field that contains a material id
        per element.
    streams_container : StreamsContainer
    data_sources : DataSources


    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.result.mapdl_material_properties()

    >>> # Make input connections
    >>> my_properties_name = str()
    >>> op.inputs.properties_name.connect(my_properties_name)
    >>> my_materials = dpf.PropertyField()
    >>> op.inputs.materials.connect(my_materials)
    >>> my_streams_container = dpf.StreamsContainer()
    >>> op.inputs.streams_container.connect(my_streams_container)
    >>> my_data_sources = dpf.DataSources()
    >>> op.inputs.data_sources.connect(my_data_sources)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.result.mapdl_material_properties(
    ...     properties_name=my_properties_name,
    ...     materials=my_materials,
    ...     streams_container=my_streams_container,
    ...     data_sources=my_data_sources,
    ... )

    >>> # Get output data
    >>> result_properties_value = op.outputs.properties_value()
    """

    def __init__(
        self,
        properties_name=None,
        materials=None,
        streams_container=None,
        data_sources=None,
        config=None,
        server=None,
    ):
        super().__init__(name="mapdl_material_properties", config=config, server=server)
        self._inputs = InputsMapdlMaterialProperties(self)
        self._outputs = OutputsMapdlMaterialProperties(self)
        if properties_name is not None:
            self.inputs.properties_name.connect(properties_name)
        if materials is not None:
            self.inputs.materials.connect(materials)
        if streams_container is not None:
            self.inputs.streams_container.connect(streams_container)
        if data_sources is not None:
            self.inputs.data_sources.connect(data_sources)

    @staticmethod
    def _spec():
        description = """Read the values of the properties of a material for a given materials
            property field (property field that contains materials
            information for each element of a mesh).It returns a
            fields container containing a field for each material
            property, with only one value per material. The following
            keys can be used: Young's modulus (keys: EX, EY, EZ),
            Poisson's ratio (keys: NUXY, NUYZ, NUXZ), Shear Modulus
            (keys: GXY, GYZ, GXZ), Coefficient of Thermal Expansion
            (keys: ALPX, ALPY, ALPZ), Volumic Mass (key: DENS), second
            Lame's coefficient (key: MU), Damping coefficient (key:
            DAMP), thermal Conductivity (keys: KXX, KYY, KZZ),
            Resistivity (keys: RSVX, RSVY, RSVZ), Specific heat in
            constant volume (key: C), Film coefficient (key: HF),
            Viscosity (key: VISC), Emissivity (key: EMIS)."""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                0: PinSpecification(
                    name="properties_name",
                    type_names=["string", "vector<string>"],
                    optional=False,
                    document="""""",
                ),
                1: PinSpecification(
                    name="materials",
                    type_names=["property_field"],
                    optional=False,
                    document="""Property field that contains a material id
        per element.""",
                ),
                3: PinSpecification(
                    name="streams_container",
                    type_names=["streams_container"],
                    optional=False,
                    document="""""",
                ),
                4: PinSpecification(
                    name="data_sources",
                    type_names=["data_sources"],
                    optional=False,
                    document="""""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="properties_value",
                    type_names=["fields_container"],
                    optional=False,
                    document="""""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server=None):
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server : server.DPFServer, optional
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.
        """
        return Operator.default_config(name="mapdl_material_properties", server=server)

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsMapdlMaterialProperties
        """
        return super().inputs

    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs : OutputsMapdlMaterialProperties
        """
        return super().outputs


class InputsMapdlMaterialProperties(_Inputs):
    """Intermediate class used to connect user inputs to
    mapdl_material_properties operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.result.mapdl_material_properties()
    >>> my_properties_name = str()
    >>> op.inputs.properties_name.connect(my_properties_name)
    >>> my_materials = dpf.PropertyField()
    >>> op.inputs.materials.connect(my_materials)
    >>> my_streams_container = dpf.StreamsContainer()
    >>> op.inputs.streams_container.connect(my_streams_container)
    >>> my_data_sources = dpf.DataSources()
    >>> op.inputs.data_sources.connect(my_data_sources)
    """

    def __init__(self, op: Operator):
        super().__init__(mapdl_material_properties._spec().inputs, op)
        self._properties_name = Input(
            mapdl_material_properties._spec().input_pin(0), 0, op, -1
        )
        self._inputs.append(self._properties_name)
        self._materials = Input(
            mapdl_material_properties._spec().input_pin(1), 1, op, -1
        )
        self._inputs.append(self._materials)
        self._streams_container = Input(
            mapdl_material_properties._spec().input_pin(3), 3, op, -1
        )
        self._inputs.append(self._streams_container)
        self._data_sources = Input(
            mapdl_material_properties._spec().input_pin(4), 4, op, -1
        )
        self._inputs.append(self._data_sources)

    @property
    def properties_name(self):
        """Allows to connect properties_name input to the operator.

        Parameters
        ----------
        my_properties_name : str

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.mapdl_material_properties()
        >>> op.inputs.properties_name.connect(my_properties_name)
        >>> # or
        >>> op.inputs.properties_name(my_properties_name)
        """
        return self._properties_name

    @property
    def materials(self):
        """Allows to connect materials input to the operator.

        Property field that contains a material id
        per element.

        Parameters
        ----------
        my_materials : PropertyField

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.mapdl_material_properties()
        >>> op.inputs.materials.connect(my_materials)
        >>> # or
        >>> op.inputs.materials(my_materials)
        """
        return self._materials

    @property
    def streams_container(self):
        """Allows to connect streams_container input to the operator.

        Parameters
        ----------
        my_streams_container : StreamsContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.mapdl_material_properties()
        >>> op.inputs.streams_container.connect(my_streams_container)
        >>> # or
        >>> op.inputs.streams_container(my_streams_container)
        """
        return self._streams_container

    @property
    def data_sources(self):
        """Allows to connect data_sources input to the operator.

        Parameters
        ----------
        my_data_sources : DataSources

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.mapdl_material_properties()
        >>> op.inputs.data_sources.connect(my_data_sources)
        >>> # or
        >>> op.inputs.data_sources(my_data_sources)
        """
        return self._data_sources


class OutputsMapdlMaterialProperties(_Outputs):
    """Intermediate class used to get outputs from
    mapdl_material_properties operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.result.mapdl_material_properties()
    >>> # Connect inputs : op.inputs. ...
    >>> result_properties_value = op.outputs.properties_value()
    """

    def __init__(self, op: Operator):
        super().__init__(mapdl_material_properties._spec().outputs, op)
        self._properties_value = Output(
            mapdl_material_properties._spec().output_pin(0), 0, op
        )
        self._outputs.append(self._properties_value)

    @property
    def properties_value(self):
        """Allows to get properties_value output of the operator

        Returns
        ----------
        my_properties_value : FieldsContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.mapdl_material_properties()
        >>> # Connect inputs : op.inputs. ...
        >>> result_properties_value = op.outputs.properties_value()
        """  # noqa: E501
        return self._properties_value



"""
rotate_in_cylindrical_cs_fc
===========================
Autogenerated DPF operator classes.
"""
from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification


class rotate_in_cylindrical_cs_fc(Operator):
    """Rotates all the fields of a fields container (not defined with a
    cynlindrical coordinate system) to its corresponding values into
    the specified cylindrical coordinate system (corresponding to the
    field position). If a coordinate system is not set in the
    coordinate_system pin, the field is rotated on each node following
    the local polar coordinate system.

    Parameters
    ----------
    field : Field or FieldsContainer
    coordinate_system : Field, optional
        3-3 rotation matrix and origin coordinates
        must be set here to define a
        coordinate system.
    mesh : MeshedRegion, optional
        Mesh support of the input fields_container,
        in case it does not have one defined.


    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.geo.rotate_in_cylindrical_cs_fc()

    >>> # Make input connections
    >>> my_field = dpf.Field()
    >>> op.inputs.field.connect(my_field)
    >>> my_coordinate_system = dpf.Field()
    >>> op.inputs.coordinate_system.connect(my_coordinate_system)
    >>> my_mesh = dpf.MeshedRegion()
    >>> op.inputs.mesh.connect(my_mesh)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.geo.rotate_in_cylindrical_cs_fc(
    ...     field=my_field,
    ...     coordinate_system=my_coordinate_system,
    ...     mesh=my_mesh,
    ... )

    >>> # Get output data
    >>> result_fields_container = op.outputs.fields_container()
    """

    def __init__(
        self, field=None, coordinate_system=None, mesh=None, config=None, server=None
    ):
        super().__init__(
            name="transform_cylindrical_cs_fc", config=config, server=server
        )
        self._inputs = InputsRotateInCylindricalCsFc(self)
        self._outputs = OutputsRotateInCylindricalCsFc(self)
        if field is not None:
            self.inputs.field.connect(field)
        if coordinate_system is not None:
            self.inputs.coordinate_system.connect(coordinate_system)
        if mesh is not None:
            self.inputs.mesh.connect(mesh)

    @staticmethod
    def _spec():
        description = """Rotates all the fields of a fields container (not defined with a
            cynlindrical coordinate system) to its corresponding
            values into the specified cylindrical coordinate system
            (corresponding to the field position). If a coordinate
            system is not set in the coordinate_system pin, the field
            is rotated on each node following the local polar
            coordinate system."""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                0: PinSpecification(
                    name="field",
                    type_names=["field", "fields_container"],
                    optional=False,
                    document="""""",
                ),
                1: PinSpecification(
                    name="coordinate_system",
                    type_names=["field"],
                    optional=True,
                    document="""3-3 rotation matrix and origin coordinates
        must be set here to define a
        coordinate system.""",
                ),
                2: PinSpecification(
                    name="mesh",
                    type_names=["abstract_meshed_region"],
                    optional=True,
                    document="""Mesh support of the input fields_container,
        in case it does not have one defined.""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="fields_container",
                    type_names=["fields_container"],
                    optional=False,
                    document="""""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server=None):
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server : server.DPFServer, optional
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.
        """
        return Operator.default_config(
            name="transform_cylindrical_cs_fc", server=server
        )

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsRotateInCylindricalCsFc
        """
        return super().inputs

    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs : OutputsRotateInCylindricalCsFc
        """
        return super().outputs


class InputsRotateInCylindricalCsFc(_Inputs):
    """Intermediate class used to connect user inputs to
    rotate_in_cylindrical_cs_fc operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.geo.rotate_in_cylindrical_cs_fc()
    >>> my_field = dpf.Field()
    >>> op.inputs.field.connect(my_field)
    >>> my_coordinate_system = dpf.Field()
    >>> op.inputs.coordinate_system.connect(my_coordinate_system)
    >>> my_mesh = dpf.MeshedRegion()
    >>> op.inputs.mesh.connect(my_mesh)
    """

    def __init__(self, op: Operator):
        super().__init__(rotate_in_cylindrical_cs_fc._spec().inputs, op)
        self._field = Input(rotate_in_cylindrical_cs_fc._spec().input_pin(0), 0, op, -1)
        self._inputs.append(self._field)
        self._coordinate_system = Input(
            rotate_in_cylindrical_cs_fc._spec().input_pin(1), 1, op, -1
        )
        self._inputs.append(self._coordinate_system)
        self._mesh = Input(rotate_in_cylindrical_cs_fc._spec().input_pin(2), 2, op, -1)
        self._inputs.append(self._mesh)

    @property
    def field(self):
        """Allows to connect field input to the operator.

        Parameters
        ----------
        my_field : Field or FieldsContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.geo.rotate_in_cylindrical_cs_fc()
        >>> op.inputs.field.connect(my_field)
        >>> # or
        >>> op.inputs.field(my_field)
        """
        return self._field

    @property
    def coordinate_system(self):
        """Allows to connect coordinate_system input to the operator.

        3-3 rotation matrix and origin coordinates
        must be set here to define a
        coordinate system.

        Parameters
        ----------
        my_coordinate_system : Field

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.geo.rotate_in_cylindrical_cs_fc()
        >>> op.inputs.coordinate_system.connect(my_coordinate_system)
        >>> # or
        >>> op.inputs.coordinate_system(my_coordinate_system)
        """
        return self._coordinate_system

    @property
    def mesh(self):
        """Allows to connect mesh input to the operator.

        Mesh support of the input fields_container,
        in case it does not have one defined.

        Parameters
        ----------
        my_mesh : MeshedRegion

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.geo.rotate_in_cylindrical_cs_fc()
        >>> op.inputs.mesh.connect(my_mesh)
        >>> # or
        >>> op.inputs.mesh(my_mesh)
        """
        return self._mesh


class OutputsRotateInCylindricalCsFc(_Outputs):
    """Intermediate class used to get outputs from
    rotate_in_cylindrical_cs_fc operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.geo.rotate_in_cylindrical_cs_fc()
    >>> # Connect inputs : op.inputs. ...
    >>> result_fields_container = op.outputs.fields_container()
    """

    def __init__(self, op: Operator):
        super().__init__(rotate_in_cylindrical_cs_fc._spec().outputs, op)
        self._fields_container = Output(
            rotate_in_cylindrical_cs_fc._spec().output_pin(0), 0, op
        )
        self._outputs.append(self._fields_container)

    @property
    def fields_container(self):
        """Allows to get fields_container output of the operator

        Returns
        ----------
        my_fields_container : FieldsContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.geo.rotate_in_cylindrical_cs_fc()
        >>> # Connect inputs : op.inputs. ...
        >>> result_fields_container = op.outputs.fields_container()
        """  # noqa: E501
        return self._fields_container
